--- ### DUNGEON ROLES

local A = select(2, ...);
local C = A.Constants;
local U = A.Utils;
local API = A.Api.Role;
local ShortsMixin = A.Mixins.Shorts;
local ShortsApiMixin = A.Mixins.ShortsApi;

local shortsStore = {};

local ItemMixin = {}
function ItemMixin:_getShortsStore()
    return shortsStore
end



-------- COMMON API -----------------

Mixin(API, ShortsApiMixin);
function API:_getShortsStore()
    return shortsStore
end
function API:GetById(id)
    return self[id];
end

-- AddShorts
-- GetByShort


function API:TryParseRoles(_roles)
    if _roles == nil or _roles == "" then return end
    local roles = U:TextSplit(_roles, C.RequestRoleSeparator);
    if #roles == 0 then return end

    local result = {};
    local found = false;
    for _, raw in pairs(roles) do
        local role = self:GetByShort(raw);
        if role then
            table.insert(result, role);
            found = true;
        end
    end
    if found then
        return result
    end
end

function API:IsDamageDealer(role)
    for _, dr in pairs(self.DDRoles) do
        if role == dr then
            return true;
        end
    end
    return false;
end

function API:HasRole(roles, checkroleId)
    local isarr = type(checkroleId) == "table" and checkroleId.id == nil;
    for _, role in pairs(roles) do
        if isarr then
            for __, one in pairs(checkroleId) do
                if role.id == one then
                    return true;
                end    
            end
        else
            if role.id == checkroleId then
                return true;
            end
        end
    end
    return false;
end

function API:ToString(roles)
    if not type(roles) == "table" then return end
    if roles.id then return roles.name end
    local txt = "";
    for _, role in pairs(roles) do
        if txt ~= "" then txt = txt .. "," end
        txt = txt .. role.name
    end
    if txt == "" then return end
    return txt;
end


function API:IsRolesHasRole(mes_roles, my_role)
    local iAmDd = self:IsDamageDealer(my_role);
    if mes_roles == nil or my_role == nil then
        --U:Log("Something was nil");
        return true
    end

    for _, rr in pairs(mes_roles) do

        if type(rr) == "table" then
            rr = rr.name;
        end

        if rr == my_role then             
            return true 
        end

        if rr == self.dd.name and iAmDd then 
            return true 
        end
    end
    return false;
end


-------- INITIALIZE -----------------

for key, name in pairs(C.Roles) do

    local item = {
        id = key,
        name = name,
        iconAtlas = C.RolesIconAtlases[key],
        shorts = { }
    }

    Mixin(item, ItemMixin, ShortsMixin);

    API[key] = item;

    item:AddShorts(name);
    item:AddShorts(unpack(C.Shorts.Roles[key]));
    
end

API.DDRoles = { API.dd.id, API.mdd.id, API.rdd.id }