local A = select(2, ...);
local U = A.Utils;
local FB = A.Lib.FB;
--local Bus = A.ChatBus;
local L = A.Localize.translate;
local Cache = A.Cache;
--local PlayersDB = A.PlayersDB;

local comp = A.Companion;

--------------------------------
------INIT----------------------

local Builders = {

    function(f)
        FB:MakeMovable(f);
        f:SetSize(135, 33);
        f:SetPoint("CENTER");  
        f:RegisterForClicks("AnyUp");
        f:SetScript("OnClick", function(self, btn) 
            if btn == "LeftButton" then
                local REQ = A:Mod("lfm-request");
                
                local remains = f:GetRequestRemains(1);
                if remains > 0 then return end;

                REQ:SendRequest();

            else
                A:ToggleMainFrame();
            end
        end);
        f:SetScript("OnEnter", function() 
            f:PopulateTooltip();
            GameTooltip:Show();
        end)        
        f:SetScript("OnLeave", function() 
            GameTooltip:Hide();
        end)        
    end,

    -- empty
    function(f)
        local t = f:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        U:UpdateFontString(t, nil, 7);
        t:SetPoint("CENTER");
        t:SetWidth(50);
        f.empty = t;
        t:SetText(U:InlineColor("Tabu-Lfg companion", "20a0ff"));
        -- t.Update = function(self)
        --     local text = "";
        --     if f._requestText then
        --         text = "";
        --     else
        --         text = L("dungeon isn't selected");
        --         -- .. " (" .. Bus.MessagesPerMinute .. "/" .. Bus.TotalMessages .. ")";
        --     end
        --     self:SetText(text);
        -- end
    end,

    -- search
    function(f)
        local t = f:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        U:UpdateFontString(t, nil, 7);
        f.search = t;
        
        t:SetWidth(f:GetWidth() - 20);
        t:SetPoint("TOPLEFT", f, "TOPLEFT", 6, -6);
        --t:SetHeight(f:GetHeight());
        t:SetJustifyH("LEFT");
        t:SetJustifyV("TOP");
        t.Update = function(self)
            self:SetText(f._requestText);
        end
        t:Hide();
    end,

    function(f)
        local t = f:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        U:UpdateFontString(t, nil, 7);
        
        f.responses = t;
        t:SetWidth(20);
        t:SetPoint("TOPRIGHT", f, "TOPRIGHT", -6, -6);
        --t:SetHeight(f:GetHeight());
        t:SetJustifyH("RIGHT");
        --t:SetText();
        t:SetTextColor(1,1,1);
        t:Hide();
        -- t.Update = function(self)
        --     self:SetText(f._requestText);
        -- end
    end,     

    function(f)
        local t = f:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        U:UpdateFontString(t, nil, 7);
        f.remains = t;
        --t:SetWidth(50);
        t:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -6, 5);
        t:SetJustifyH("RIGHT");
        --t:SetText("");
        t:SetTextColor(1,.4,0);
        t:Hide();
        t.Update = function(self)
            local remains = ceil(f:GetRequestRemains(1));
            local text = remains .. "s";
            if remains == 0 then
                text = L("click to send");
            end
            self:SetText(text);
        end
        --t:SetScript("OnUpdate", t.OnUpdate);
    end,     

    function(f)
        f:SetScript("OnUpdate", function(self)            
            --self.stats:Update();
            self.remains:Update();
            local num = self:GetRequestRemains("nozone");
            if num == "nozone" then
                self.search:SetTextColor(1, .8, 0);
            elseif num == 0 then
                self.search:SetTextColor(.3, 1, .4);
            else
                self.search:SetTextColor(.7, .7, .7);
            end
        end)
        
    end

}

FB:ApplyFrameBuilders(comp, Builders);


--------------------------------
------MIXINS----------------------


function comp:Update(entries)
    local req = A:Mod("lfm-request"); --A:GetLfmReq();
    local zone = req:GetZone();
    if not zone then
        self._requestText = nil;
        self.empty:Show();
        self.search:Hide();
        self.remains:Hide();
        self.responses:Hide();
        -- self.stats:SetJustifyH("CENTER");
        -- self.stats:ClearAllPoints();
        -- self.stats:SetPoint("CENTER");
        -- self.stats:SetWidth(50);
        
    else
        -- local data = { zone:ToString(), req:GetRolesText(), req:GetNote() };
        -- self._requestText = U:JoinToString(data, "\n");

        self._requestText = U:InlineColor(zone:ToString(), zone:GetColor());
        
        local roles = req:GetRolesText();
        if roles and roles ~= "" then
            self._requestText = self._requestText .. "\n" .. roles;
        end

        local note = req:GetNote();
        if note and note ~= "" then
            self._requestText = self._requestText .. "\n" .. note;
        end

        --print(" -- ", self._requestText, roles, note);

        self.empty:Hide();
        self.search:Show();
        self.remains:Show();
        self.responses:Show();
        -- self.stats:SetJustifyH("RIGHT");
        -- self.stats:ClearAllPoints();
        -- self.stats:SetPoint("RIGHT", -5, 0);
        -- self.stats:SetWidth(20);
        if type(entries) == "table" then
            self.responses:SetText(#entries);
        end
        self.search:Update();
    end    

    --print(" companion update")
end

function comp:PopulateTooltip()
    local PlayersDB = A:Mod("players-db");
    local Bus = A:Mod("chat-bus");

    GameTooltip:SetOwner(self);
    GameTooltip:AddLine(U:InlineColor("Tabu-Lfg Companion", "ffaa00"));
    GameTooltip:AddLine(L("Unthrottled/throttled Chat messages during this session") .. ": " .. U:InlineColor(Bus.TotalMessages, "ffffff") .. " / " .. U:InlineColor(Bus.TotalThrottledMessages, "ddffdd"));
    GameTooltip:AddLine(L("Unthrottled/throttled Message per minute") .. ": " .. U:InlineColor(Bus.MessagesPerMinute, "00ccff") .. " / " .. U:InlineColor(Bus.ThrottledMessagesPerMinute, "00aaff"));
    GameTooltip:AddLine(L("Players met during this session") .. ": " .. U:InlineColor(Bus.TotalPinged, "ffffff"));
    
    if A.BUILD.debugEnabled then
        GameTooltip:AddLine(L("Tabu-Lfg players today") .. ": " .. U:InlineColor(Bus.PingResponses, "ffffff"));
    end
    GameTooltip:AddLine(L("Total Tabu-Lfg players met") .. ": " .. U:InlineColor(PlayersDB:GetAddonUsersCount(), "00ccff"));

    GameTooltip:AddLine("---");
    GameTooltip:AddLine(U:InlineColor(L("Left Click"),"0080ff") .. " - " .. U:InlineColor(L("Sends LFM message to LFG chat"),"ffffff"));
    GameTooltip:AddLine(U:InlineColor(L("Right Click"),"0080ff") .. " - " .. U:InlineColor(L("For open or hide Tabu-Lfg addon"),"ffffff"));


end

function comp:GetRequestRemains(def)
    local REQ = A:Mod("lfm-request"); --A:GetLfmReq();
    if not REQ:HasZone() then return def end;
    local remains = REQ:GetSecondsToNewRequest();
    if remains > 0 then return remains end;
    return 0;
end

function comp:Toggle()
    local f = self;
    if f:IsShown() then
        f:Hide();
    else
        f:Show();
    end
end




