-- chat
local A = select(2, ...);
local mod = A:Mod("chat-frame");
local fb = A.Lib.FB;
local U = A.Utils;
local L = A.Localize.translate;
local C = A.Constants;

local DATA = A:Mod("chat-data");
local REQ = A:Mod("lfm-request");

local LOC = {
    SelectDungeon = L("Select dungeon"),
    NotSpecified = L("Not Specified"),
    Close = L("Close"),
    Dungeons = L("Dungeons"),
    Mode = L("Mode"),
    Era = L("Addon"),
    Send = L("Send"),
    Invite = L("Invite"),
    Whisper = L("Whisper"),
    ClearZone = L("Unselect dungeon"),
    DungeonNotSelected = L("Dungeon not selected"),
    DungeonNote = L("Note"),
    EditNote = L("Edit dungeon note"),
    Change = L("Change"),
    Roles = L("Roles"),
    RolesTooltip = L("Specify roles you are looking for selected dungeon"),
    NoteTooltip = L("Specify additional information for selected dungeon"),
    SendButton_NotSpecifiedDungeon = L("Please select Dungeon you are looking for for sending a message in LFG channel"),
}


local function GetWeakFilterDropdownMenuList()

    local menu = {}
    local click, checked;

    table.insert(menu, { text = L("Show dungeons"), notCheckable = true, isTitle = true });

    click = function(self, arg1, arg2, checked) 
        local req = REQ;
        local val = req.weakFilter.eras[self.value];
        if val == nil then val = true end
        req.weakFilter.eras[self.value] = not val;
        req:OnFilterChange();
    end

    checked = function(self)
        return REQ.weakFilter.eras[self.value] ~= false;
    end

    for key, eraName in pairs(C.Eras) do
        table.insert(menu, { text = eraName, value = eraName, func = click, checked = checked, keepShownOnClick = true });
    end

    --table.insert(menu, { separator = true });
    --table.insert(menu, { text = "Dungeon mode", notCheckable = true, isTitle = true });

    click = function(self, arg1, arg2, checked) 
        local req = REQ;
        local val = req.weakFilter.modes[self.value];
        if val == nil then val = true end
        req.weakFilter.modes[self.value] = not val;
        req:OnFilterChange();
    end

    checked = function(self)
        return REQ.weakFilter.modes[self.value] ~= false;
    end

    for key, mode in pairs(C.Modes) do
        table.insert(menu, { text = mode[2], value = mode[1], func = click, checked = checked, keepShownOnClick = true });
    end
    table.insert(menu, { text = L("From my LFG list only"), value = true, keepShownOnClick = true, func = function(self, a1,a2, checked) 
        local req = REQ;
        local val = req.weakFilter.inMyWishList;
        if val == nil then val = false end
        req.weakFilter.inMyWishList = not val;
        req:OnFilterChange();
    end, checked = function(self) 
        return REQ.weakFilter.inMyWishList == true;
    end });

    table.insert(menu, { separator = true });

    table.insert(menu, { text = L("Show quests"), value = true, keepShownOnClick = true, func = function(self, a1,a2,checked) 
        local req = REQ;
        local val = req.weakFilter.showQuests;
        if val == nil then val = true end
        req.weakFilter.showQuests = not val;
        req:OnFilterChange();
    end, checked = function(self) return REQ.weakFilter.showQuests ~= false   end });

    table.insert(menu, { text = L("Show undetected"), value = true, keepShownOnClick = true, func = function(self, a1,a2,checked) 
        local req = REQ;
        local val = req.weakFilter.showUndetected;
        if val == nil then val = true end
        req.weakFilter.showUndetected = not val;
        req:OnFilterChange();
    end, checked = function(self) return REQ.weakFilter.showUndetected ~= false   end });

    table.insert(menu, { separator = true });
    table.insert(menu, { text = LOC.Close, notCheckable = true });

    return menu;
end


local FrameMixin = {
    OnShow = function(self) 
        REQ.weakFilter.myWishList = A:Mod("lfg-db"):GetCheckedHash();
        self.faux:UpdateButtons();
    end,
}

local FrameBuilders = {

    -- initial
    function(f)
        f:Hide();
        f.frameName = "ChatFrame";
        Mixin(f, FrameMixin);
    end,

    -- faux scroll
    function(f)

        local faux = CreateFrame("Frame", "$parent_Holder", f);
        f.faux = faux;
        faux:SetPoint("TOPLEFT", f, "TOPLEFT", 8, -63);
        faux:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -3, 28);

        local buttonsCount = 15;
    
        U:AddFauxScroll(faux, buttonsCount, A:Com("chat-faux-button").CreateButton, {
    
            GetData = function(self)
                return DATA:Get(REQ.weakFilter);
            end,
    
            UpdateButtonByItem = function(self, item)
                if not item or not item.__button_index then return end;
                local btn = self:GetButton(item.__button_index);
                if not btn or btn.data ~= item then return end;
                btn:Update(item);
            end,
        });

    end,

    -- scripts and handlers
    function(f)
        f:SetScript("OnShow", f.OnShow);
        f:SetScript("OnHide", f.OnHide);
    
        local ChangeHandler = function(event, player) 
            if not f:IsShown() then return end        
            f.faux:UpdateButtons();
        end
    
        DATA:On("player:update", ChangeHandler, f);
        REQ:On("filter:change", ChangeHandler, f);        
    end,

    -- weak filter
    function(f)
        --local btn = f.faux.Buttons[1];
        local b = CreateFrame("Button", "$parent_WeakFilter", f);
        b:SetPoint("TOPLEFT", f, "TOPLEFT", 60, -32);
        b:SetSize(120, 20);
        fb:AddHoverBackground(b, {0,0,0,.5});
        local arrowBtn = U:CreateArrowButton("$parent_ArrowButton", b, { "LEFT", b, "LEFT", 0, 0 }, 16);

        local t = b:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        --t:SetAllPoints();
        t:SetPoint("LEFT", arrowBtn, "RIGHT", 5, 0);

        t:SetText(L("Setup filters"));
        t:SetJustifyH("LEFT");
        --print("# btn", btn);
        local dropdown = CreateFrame("Frame", "$parent_Dropdown", b, "UIDropDownMenuTemplate");
        b:SetScript("OnClick", function(self) 

            PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
            local menu = GetWeakFilterDropdownMenuList();
            U:EasyMenu(menu, dropdown, b, 0, 0, "MENU");
            --EasyMenu()
        end);

        return b;
    end,
}

function mod.CreateFrame(parent, id)

    local frame = CreateFrame("Frame", "$parent_Tab_ChatFrame", parent);

    fb:ApplyFrameBuilders(frame, FrameBuilders);

    return frame;
end

function mod:GetTabInfo()
    return {
        text = L("Chat"),
        frameHeader = L("Chat activity"),
        tabFrameBuilder = mod.CreateFrame
    }
end