-- chat
local A = select(2, ...);
local com = A:Com("chat-faux-button");
local fb = A.Lib.FB;
local U = A.Utils;
local C = A.Constants;
local RoleApi = A.Api.Role;
local TypeApi = A.Api.Type;
local L = A.Localize.translate;

local HUNDRED_SECONDS = 100;
local HUNDRED_MINUTES = 100 * 60;
local SECONDS_IN_HOUR = 60 * 60;

local Inviter = A:Com("send-invite");


local function RoleIconTexture_Update(self, roles)
    if not roles or #roles == 0 or not RoleApi:HasRole(roles, self.roleId) then
        self:SetTexture(nil);
    else
        self:SetAtlas(self.atlasId)
    end
end

local function CreateRoleIconTexture(parent, prev, roleId, atlasId)
    local t = parent:CreateTexture(nil, "OVERLAY");
    if prev == nil then
        t:SetPoint("LEFT", parent, "LEFT", 0, 0);
    else
        t:SetPoint("LEFT", prev, "RIGHT", 0, 0);
    end
    t.sizePx = 20;
    t.roleId = roleId;
    t.atlasId = atlasId; --"groupfinder-icon-role-large-tank";
    t:SetSize(t.sizePx, t.sizePx);
    t.Update = RoleIconTexture_Update;
    return t;
end



local ButtonConfig = {
    textures = {
        {
            parentKey = "texLeft",
            texture = "Interface\\AuctionFrame\\UI-AuctionItemNameFrame",
            point = { "LEFT", "parent", "LEFT", 2, 0 },
            width = 10,
            height = 25,
            texCoord = { 0, 0.078125, 0, 1.0 }
        },
        {
            parentKey = "texRight",
            texture = "Interface\\AuctionFrame\\UI-AuctionItemNameFrame",
            point = { "RIGHT", "parent", "RIGHT", 0, 0 },
            width = 10,
            height = 25,
            texCoord = { 0.75, 0.828125, 0, 1.0 }
        },
        {
            parentKey = "texMid",
            texture = "Interface\\AuctionFrame\\UI-AuctionItemNameFrame",
            points = {{ "LEFT", "parent", "LEFT", 12, 0 }, { "RIGHT", "parent", "RIGHT", -10, 0 }},
            width = 10,
            height = 25,
            texCoord = { 0.078125, 0.75, 0, 1.0 }
        },
        {
            parentKey = "icon",
            layer = "BORDER",
            point = { "LEFT" },
            width = 32,
            height = 32,
        },
        {
            parentKey = "iconBorder",
            texture = "Interface\\Common\\WhiteIconFrame",
            point = { "LEFT" },
            width = 32,
            height = 32,
            hidden = true,
        },
    }
}

local ButtonMixin = {
    Update = function(self, data)

        self.data = data;

        -- New
        for _, item in pairs(self.items) do
            if type(item) == "function" then
                item();
            elseif item.Update then
                item:Update();
            end
        end

        if self.data and self.data:IsDetected() then
            self.dungeon:Show();
            self.roles:Show();
            self.note:ReAttach(self.roles);
        else
            self.dungeon:Hide();
            self.roles:Hide();
            self.note:ReAttach(self.typeIcon);
        end

    end,

    UpdateRole = function(self)
        -- if self.data.role then
        --     self.role:SetText(self.data.role);
        -- elseif self.data.roles then
        --     local txt = U:JoinToString(self.data.roles, ", ");
        --     self.role:SetText(txt);
        -- else
        --     self.role:SetText("");
        -- end
    end,

    UpdateColor = function(self)
        -- local r, g, b = GetClassColor(self.data.player.class);
        -- self.name:SetTextColor(r,g,b);

        -- if self.data.native then
        --     r,g,b = 0,1,.3;
        -- else
        --     r,g,b = 1,1,1;
        -- end
        -- self.note:SetTextColor(r,g,b);
        --self.class:SetTextColor(r,g,b);
    end,

    UpdateTime = function(self)

        if not self.data or not self.data.time then
            self.time:SetText("");
            return;
        end

        local text = "";
        local secondsAgo = floor(GetTime() - self.data.time);

        if secondsAgo < 100 then
            text = "-" .. secondsAgo .. "s"
        elseif secondsAgo < HUNDRED_MINUTES then
            local minutesAgo = ceil(secondsAgo / 60);
            text = "-" .. minutesAgo .. "m";
        else
            local hoursAgo = ceil(secondsAgo / SECONDS_IN_HOUR);
            text = "-" .. hoursAgo .. "m";
        end

        self.time:SetText(text);
    end,

    OnFrameUpdate = function(self, elapsed)
        self._lastFrameUpdate = self._lastFrameUpdate + elapsed;
        if self._lastFrameUpdate <= self._frameUpdateInterval then return end;

        self:UpdateTime();

        self._lastFrameUpdate = 0;
    end,

    PopulateTooltip = function(self)
        if not self.data then return end
        local pl = self.data.player;
        local isNative = pl.native == true;

        GameTooltip:SetOwner(self);
        local topText = self.data.text or ""
        if not isNative then
            topText = pl.name .. ": " .. topText;
        end

        GameTooltip:AddLine(topText, nil, nil, nil, true);

        if isNative then
            GameTooltip:AddLine("---");
            
            local role = self.data.role and self.data.role.name or "";

            U:InlineColor()
            local player_text = pl.name or "";
            local clscolor; 

            if pl.class then
                --local _,_,_, rgb = select(4, GetClassColor(pl.class));
                clscolor = select(4, GetClassColor(pl.class));
            end

            if pl.level then
                player_text = pl.level .. " " .. player_text;                
            end

            GameTooltip:AddDoubleLine(U:InlineColor(player_text, clscolor), U:InlineColor(role, "50ff30"))
            GameTooltip:AddDoubleLine(L("Item level:"), pl.avgItemLevel or "");
            GameTooltip:AddDoubleLine(L("Talents:"), pl.talents or "");
            GameTooltip:AddLine(L("Note") .. ": " .. (self.data.note or ""));

        end
        GameTooltip:AddLine("---");
        GameTooltip:AddLine(U:InlineColor(L("Left Click"),"0080ff") .. " - " .. U:InlineColor(L("Send whisper to player"),"ffffff"));
        GameTooltip:AddLine(U:InlineColor(L("Right Click"),"0080ff") .. " - " .. U:InlineColor(L("Send invite to player"),"ffffff"));

    end,

    OnMouseEnter = function(self)
        self:PopulateTooltip();
        GameTooltip:Show();
    end,


    OnMouseLeave = function(self)
        GameTooltip:Hide();
    end,

    OnClick = function(self, button)
        local isShift = IsShiftKeyDown();
        local isAlt = IsAltKeyDown();
        if not isShift and button == "LeftButton" then
            ChatFrame_OpenChat("/w " .. self.data.player.name .. " ");
            return;
        end

        if not isShift and button == "RightButton" then
            local userName = self.data.player.name;
            Inviter:Send(userName)
            return;
        end


        if isShift and isAlt and button == "LeftButton" and Tabu then
            Tabu.dump(self.data);
        end
    end,

    OnShow = function(self)
        self._lastFrameUpdate = 0; 
        self._frameUpdateInterval = 0.5;
    end,
}

local ButtonBuilders = {

    -- init
    function(f)
        local parent = f:GetParent();
        fb:SetupTextures(f, ButtonConfig, parent);
        f:RegisterForClicks("AnyUp");
        f:SetHeight(27);

        Mixin(f, ButtonMixin);
    end,



    -- scripts and handlers
    function(f)
        f:SetScript("OnUpdate", f.OnFrameUpdate);
        f:SetScript("OnShow", f.OnShow);
        f:SetScript("OnEnter", f.OnMouseEnter);
        f:SetScript("OnLeave", f.OnMouseLeave);
        f:SetScript("OnClick", f.OnClick);    
    end,


    -- items to update
    function(b)
        b.items = {};
    end,

    -- time
    function(b)

        local t = b:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        U:UpdateFontString(t, nil, 8);
        t:SetPoint("LEFT", b, "LEFT", 5, 0);
        t:SetWidth(30);
        b.time = t;
        return t;

    end,

    -- name
    function(b, prev)
        local t = b:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");

        b.name = t;
        t.originalWidth = 90;
        t:SetSize(t.originalWidth, 20);
        t:SetPoint("LEFT", prev, "RIGHT", 5, 0);
        t:SetJustifyH("LEFT");

        t.Update = function(self)

            if not b.data or not b.data.player then
                self:SetText("")
                return;
            end

            local plr = b.data.player;

            self:SetTextColor(GetClassColor(plr.class));

            local name = plr.name or "";
            local level = plr.level;
            level = level and (level .. " ") or "";

            self:SetText(level .. name);
        end
        table.insert(b.items, t);
        return t;
    end,   
    
    -- LFM or LFG Icon 
    -- .typeIcon
    function(b, prev)
        local t = b:CreateTexture(nil, "OVERLAY");
        t:SetPoint("LEFT", prev, "RIGHT", 5, 0);
        b.typeIcon = t;

        t:SetSize(20, 20);
        
        t.Update = function(self)
            local typeId = b.data.type and b.data.type.id or nil;
            local roleIcon = b.data.role and b.data.role.iconAtlas or nil;
            if b.data.native and roleIcon then
                self:SetAtlas(roleIcon);
            elseif typeId == TypeApi.Lfm.id then
                self:SetAtlas("socialqueuing-icon-group");
            elseif typeId == TypeApi.Lfg.id then
                self:SetAtlas(roleIcon or "groupfinder-icon-friend");
            else
                self:SetTexture(nil);
            end
            
        end

        table.insert(b.items, t);
        return t;
    end,

    -- Dungeon/quest name
    function(b, prev)
        local t = b:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        b.dungeon = t;
        --U:UpdateFontString(t, nil, 8);
        t:SetPoint("LEFT", prev, "RIGHT", 5, 0);
        t:SetWidth(150);
        t:SetJustifyH("LEFT");
        t.Update = function(self)
            local dun = b.data.dungeon;
            local qst = b.data.quest;

            if qst then
                self:SetText(qst.name);
                self:SetTextColor(1,1,0);
            elseif dun then
                local dungeonColor = dun:GetColor(); --C.Colors.dungeon:GetColor(dun); --MODECOLORSTABLE[dun:IsHeroic()];
                self:SetText(dun:ToString());
                self:SetTextColor(unpack(dungeonColor));
            else
                self:SetText("");
                self:SetTextColor(1,1,1);
            end

        end

        table.insert(b.items, t);
        return t;
    end,

    -- Roles
    function(b, prev)
        local f = CreateFrame("Frame", nil, b);
        f:SetSize(60, 20);
        f:SetPoint("LEFT", prev, "RIGHT", 5, 0);
        
        b.roles = f;

        local tank = CreateRoleIconTexture(f, nil, RoleApi.tank.id, "groupfinder-icon-role-large-tank");
        local heal = CreateRoleIconTexture(f, tank, RoleApi.heal.id, "groupfinder-icon-role-large-heal");
        local dd = CreateRoleIconTexture(f, heal, { RoleApi.dd.id, RoleApi.rdd.id, RoleApi.mdd.id,  }, "groupfinder-icon-role-large-dps");
        f.Update = function()
            local roles = not b.data.role and b.data.roles or nil;
            tank:Update(roles);
            heal:Update(roles);
            dd:Update(roles);
        end

        table.insert(b.items, f);
        return f;
    end,

    -- note
    function(b, prev)
        local t = b:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        t:SetTextColor(1,1,1);
        b.note = t;
        t:SetPoint("LEFT", prev, "RIGHT", 5, 0);
        t:SetPoint("RIGHT", b, "RIGHT", -5, 0);
        t:SetHeight(15);
        t:SetJustifyH("LEFT");

        t.Update = function(self)
            if not b.data then
                self:SetText("")
                return;
            end

            local noteText = b.data.note or b.data.autoNote;
            --local text = noteText or b.data.text;


            if b.data:IsDetected() then
                if b.data.quest then
                    self:SetText(b.data.quest.text);
                else
                    self:SetText(noteText);
                end
            else
                self:SetText(b.data.text);
            end
        end

        t.ReAttach = function(self, prevEl)
            t:ClearAllPoints();
            t:SetPoint("LEFT", prevEl, "RIGHT", 5, 0);
            t:SetPoint("RIGHT", b, "RIGHT", -5, 0);
        end

        table.insert(b.items, t);
        return t;
    end,  


}

-- setup points
local function SetupPoint(f, index, parent, prev)

    if index == 1 then
        f:SetPoint("TOPLEFT", parent, "TOPLEFT", 0, 0);
        f:SetPoint("TOPRIGHT", parent, "TOPRIGHT", -32, 0);
    else
        f:SetPoint("TOPLEFT", prev, "BOTTOMLEFT");
        f:SetPoint("TOPRIGHT", prev, "BOTTOMRIGHT");
    end
end

function com.CreateButton(index, parent, prev)

    local f = CreateFrame("Button", "$parent_Btn" .. index, parent);
    f:SetID(index);
    SetupPoint(f, index, parent, prev);

    fb:ApplyFrameBuilders(f, ButtonBuilders);


    return f;
end