local A = select(2, ...)
local mod = A:Mod("chat-frame");
local L = A.Localize.translate;
local Cache = A.Cache;


local Options = {

    {
        type = "GroupName",
        name = "ChatMessageObsolescenceHeader",
        header = L("Chat message obsolescence")
    },
    {
        type = "Number",
        name = "ChatMessageObsolescence_dungeon",
        text = L("<dungeon>"),
        tooltip = L("Each chat entry with detected dungeon will be considered obsolete after this amount of seconds. Enter 0 for disable obsolescence"),
        minValue = 0,
        get = function()
            return Cache:GetOption("chat.entry.isOldAfter.dungeon")
        end,
        set = function(value)
            Cache:SetOption("chat.entry.isOldAfter.dungeon", value)
        end
    },
    {
        type = "Number",
        name = "ChatMessageObsolescence_dungeon_lfg",
        text = L("<lfg dungeon>"),
        tooltip = L("Each chat entry with detected dungeon of type lfg will be considered obsolete after this amount of seconds. Enter 0 for disable obsolescence"),
        minValue = 0,
        get = function()
            return Cache:GetOption("chat.entry.isOldAfter.dungeon.Lfg")
        end,
        set = function(value)
            Cache:SetOption("chat.entry.isOldAfter.dungeon.Lfg", value)
        end
    },    
    {
        type = "Number",
        name = "ChatMessageObsolescence_dungeon_lfm",
        text = L("<lfm dungeon>"),
        tooltip = L("Each chat entry with detected dungeon of type lfm will be considered obsolete after this amount of seconds. Enter 0 for disable obsolescence"),
        minValue = 0,
        get = function()
            return Cache:GetOption("chat.entry.isOldAfter.dungeon.Lfm")
        end,
        set = function(value)
            Cache:SetOption("chat.entry.isOldAfter.dungeon.Lfm", value)
        end
    },    
    {
        type = "Number",
        name = "ChatMessageObsolescence_quest",
        text = L("<quest>"),
        tooltip = L("Each chat entry with detected quest will be considered obsolete after this amount of seconds. Enter 0 for disable obsolescence"),
        minValue = 0,
        get = function()
            return Cache:GetOption("chat.entry.isOldAfter.quest")
        end,
        set = function(value)
            Cache:SetOption("chat.entry.isOldAfter.quest", value)
        end
    },    
    {
        type = "Number",
        name = "ChatMessageObsolescence_undetected",
        text = L("<undetected>"),
        tooltip = L("Each undetected chat entry will be considered obsolete after this amount of seconds. Enter 0 for disable obsolescence"),
        minValue = 0,
        get = function()
            return Cache:GetOption("chat.entry.isOldAfter.undetected")
        end,
        set = function(value)
            Cache:SetOption("chat.entry.isOldAfter.undetected", value)
        end
    },    
    {
        type = "GroupName",
        name = "ChatMessageSendHeader",
        header = L("Sending chat message")
    },
    {
        type = "Checkbox",
        name = "ChatMessageSend_toLFG",
        text = L("Send chat message to the LFG channel"),
        tooltip = L("When enabled sends generated message to the LFG channel"),
        checked = function() 
            return Cache:GetOption("chat.send.toLfg");
        end,
        click = function() 
            local value = not Cache:GetOption("chat.send.toLfg");
            Cache:SetOption("chat.send.toLfg", value);
        end,
    },
    {
        type = "Checkbox",
        name = "ChatMessageSend_LfgSpread",
        text = L("Send multiple messages to LFG channel"),
        tooltip = L("When enabled will send several messages into the LFG channel instead of one, the amount of messages will be calculated automaticaly base on current chat speed. Will not take effect if Sending to LFG channel disabled"),
        checked = function() 
            return Cache:GetOption("chat.send.LfgSpread");
        end,
        click = function() 
            local value = not Cache:GetOption("chat.send.LfgSpread");
            Cache:SetOption("chat.send.LfgSpread", value);
        end,
    },
    {
        type = "Checkbox",
        name = "ChatMessageSend_toGeneral",
        text = L("Send message to General channel"),
        tooltip = L("When enabled will send One message into the General channel"),
        checked = function() 
            return Cache:GetOption("chat.send.toGeneral");
        end,
        click = function() 
            local value = not Cache:GetOption("chat.send.toGeneral");
            Cache:SetOption("chat.send.toGeneral", value);
        end,
    },
    {
        type = "Checkbox",
        name = "ChatMessageSend_addAngleBrackets",
        text = L("Wrap important words with angle brackets"),
        tooltip = L("When enabled will wrap dungeon name, tank and heal roless with angle brackets."),
        checked = function() 
            return Cache:GetOption("chat.send.addAngleBrackets");
        end,
        click = function() 
            local value = not Cache:GetOption("chat.send.addAngleBrackets");
            Cache:SetOption("chat.send.addAngleBrackets", value);
        end,
    },
    {
        type = "GroupName",
        name = "ChatThrottlingOptions",
        header = L("Chat throttling")
    },
    {
        type = "Checkbox",
        name = "ChatThrottler",
        text = L("Use chat throttling"),
        tooltip = L("When enabled improve performance by droping off some repeated chat entries"),
        checked = function() 
            return Cache:GetOption("chat.throttler");
        end,
        click = function() 
            local value = not Cache:GetOption("chat.throttler");
            Cache:SetOption("chat.throttler", value);
            A:Mod("chat-bus"):Restart();
        end,
    },
    {
        type = "Number",
        name = "ChatThrottler_StoreFor",
        text = L("Throttling value in seconds"),
        tooltip = L("Each chat entry will be considered a duplicate for this amount of seconds"),
        minValue = 0,
        get = function()
            return Cache:GetOption("chat.throttler.storeFore")
        end,
        set = function(value)
            Cache:SetOption("chat.throttler.storeFore", value)
        end
    },    
}


function mod:GetConfigOptions()
    return Options;
end