local A = select(2, ...);
local com = A:Com("lfg-faux-button");
local fb = A.Lib.FB;
local U = A.Utils;
local Cache = A.Cache;
local L = A.Localize.translate;
local C = A.Constants;
local mod = A:Mod("lfg-frame");

local SEND_DELAY_IN_SECONDS = A:Mod("config").SEND_DELAY_IN_SECONDS;


local LOC = {
    SelectDungeon = L("Select dungeon"),
    NotSpecified = L("Not Specified"),
    Empty = L("---"),
    Close = L("Close"),
    Role = L("Specify your role"),
    DefaultRole = L("Your role"),
    DefaultRoleTooltip = L("Your default role, this role will be used for all dungeons with unspecified role"),
    DefaultNote = L("Your note"),
    DefaultNoteTooltip = L("Your default note, this note will be used for all dungeons with unspecified note"),
    DungeonRoleTooltip = L("You may specify special role for this dungeon"),
    DungeonNoteTooltip = L("You may specify special note for this dungeon"),
    EditNote = L("Edit note"), 
    Change = L("Change"),
    Send = L("Send"),
    CheckedExplain = L("When checked will autoresponse to a player who send a message in an LFG channel about this dungeon")
}


local function OnButtonChecked(self)
    local parent = self:GetParent();
    parent:UpdateChecked(self:GetChecked());
end

local function OnSendRequestClicked(self)
    local f = self:GetParent();
    local p = f:GetParent();
    p.lastClicked = GetTime();
    p:GetParent():SendRequest(f.data);
end

local function SendRequestButton_Update(self) 
    local f = self:GetParent();
    local p = f:GetParent();
    local t = GetTime();
    local d = t - p.lastClicked;
    local remains = SEND_DELAY_IN_SECONDS - d;
    if remains > 0 then
        self:SetText(floor(remains));
        self:SetEnabled(false);
    else
        self:SetText(self.enabledText);
        self:SetEnabled(true);
    end
end

local function OnButtonEditNote(value, button)
    button.data.note = value;
    Cache:SetOption("lfg.dungeon."..button.data.id..".note", value);
    button:UpdateNote();
end





local function FauxButtonDropdownRole_Click(self, arg1, arg2, checked)
    local v = self.value ~= "" and self.value or nil;
    local data = arg1;
    data.role = v;
    Cache:SetOption("lfg.dungeon."..data.id..".role", v);
    arg2:UpdateRole();
end

local function FauxButtonDropdownRole_Checked(data, role)
    return data and data.role == role;
end

local function OpenFauxButtonRoleDropdown(self) 
    PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
    local menuList = mod:GetRolesDropdownMenu(self, FauxButtonDropdownRole_Click, FauxButtonDropdownRole_Checked);
    U:EasyMenu(menuList, self.dropdown, self, 0, 0, "MENU");
end


local ButtonConfig = {
    textures = {
        {
            parentKey = "texLeft",
            texture = "Interface\\AuctionFrame\\UI-AuctionItemNameFrame",
            point = { "LEFT", "parent", "LEFT", 2, 0 },
            width = 10,
            height = 20,
            texCoord = { 0, 0.078125, 0, 1.0 }
        },
        {
            parentKey = "texRight",
            texture = "Interface\\AuctionFrame\\UI-AuctionItemNameFrame",
            point = { "RIGHT", "parent", "RIGHT", 0, 0 },
            width = 10,
            height = 20,
            texCoord = { 0.75, 0.828125, 0, 1.0 }
        },
        {
            parentKey = "texMid",
            texture = "Interface\\AuctionFrame\\UI-AuctionItemNameFrame",
            points = {{ "LEFT", "parent", "LEFT", 12, 0 }, { "RIGHT", "parent", "RIGHT", -10, 0 }},
            width = 10,
            height = 20,
            texCoord = { 0.078125, 0.75, 0, 1.0 }
        },
    }
}

local ButtonMixin = {
    Update = function(self, data)
        self.data = data;
        data = data or {};

        local name = data.name or "";
        local mode = U:GetModeName(data.mode);
        if name ~= ""  and mode == C.Modes.Heroic[2] then
            name = name .. " - " .. mode;
        end

        self.zone:SetText(name);

        self:UpdateRole();
        self:UpdateChecked();
        self:UpdateNote();
        self:UpdateColors();

    end,

    UpdateRole = function(self)

        local role = self.data and self.data.role or LOC.Empty;
        self.role:SetText(role);

    end,

    UpdateNote = function(self)
        self.note:SetText(self.data and self.data.note or "");
    end,

    UpdateColors = function(self)
        if self.data == nil then return end;
        local color = self.data:GetColor();
        self.zone:SetTextColor(unpack(color));
    end,

    IsChecked = function(self)
        return self.data and self.data.checked == true;
    end,

    UpdateChecked = function(self, newValue)
        local checked;
        if newValue == nil then
            checked = self:IsChecked();
        else
            checked = newValue;
            if self.data then
                self.data.checked = newValue;
                Cache:SetOption("lfg.dungeon."..self.data.id..".checked", newValue);
            end
        end
        self.checked:SetChecked(checked);


        if checked and self.data ~= nil then
            self.send:Show();
            self.texLeft:Show();
            self.texMid:Show();
            self.texRight:Show();
        else
            self.send:Hide();
            self.texLeft:Hide();
            self.texMid:Hide();
            self.texRight:Hide();

        end

    end,

    ShowSendTooltip = function(self)
        local text = self:GetParent():GetParent():GetRequestText(self.data);
        U:WillBeSentTooltip(self.send, text);
    end,

}

local ButtonBuilders = {

    -- set lastClicked on parent
    function(b)
        local p = b:GetParent()
        if not p.lastClicked then
            p.lastClicked = GetTime() - (SEND_DELAY_IN_SECONDS);
        end
    end,

    -- check button
    function(b)
        local check = CreateFrame("CheckButton", "$parent_Enabled", b, "ChatConfigCheckButtonTemplate");
        fb:SetTooltip(check, LOC.CheckedExplain)
        b.checked = check;

        check:SetPoint("LEFT", b, "LEFT", 0, 0);
        
        check:SetHitRectInsets(0, -220, 0, 0);

        check:SetScript("OnClick", OnButtonChecked);

        return check;
    end,

    -- zone
    function(b, prev)
        local t = b:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");

        b.zone = t;
        t:SetSize(300, 20);
        t:SetPoint("LEFT", prev, "RIGHT", 5, 0);
        t:SetText("text")
        t:SetJustifyH("LEFT");

        return t;
    end,

    -- role
    function(b, prev)
        local f = CreateFrame("Button", "$parent_RoleHld", b);

        
        local parent = b:GetParent();

        f:SetSize(30, 20);
        f:SetPoint("LEFT", prev, "RIGHT", 5, 0);

        fb:AddHoverBackground(f, {0,0,0,.5})

        fb:SetTooltip(f, LOC.DungeonRoleTooltip);

        local dropdown = CreateFrame("Frame", "$parent_RoleDropdown", f, "UIDropDownMenuTemplate");
        f.dropdown = dropdown;
        f:SetScript("OnClick", OpenFauxButtonRoleDropdown);

        local t = f:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");

        t:SetAllPoints();
        b.role = t;

        return t;

    end,

    -- send request button
    function(f, prev)

        local b = CreateFrame("Button", "$parent_SendRequest", f, "UIPanelButtonTemplate");
        f.send = b;
        
        fb:SetTooltip(b, function() f:ShowSendTooltip() end);

        b.enabledText = LOC.Send;
        b:SetText(b.enabledText);
        b:SetPoint("RIGHT", f, "RIGHT", 0, 0);
        b:SetWidth(70);
        b:SetHeight(20);
        U:UpdateFontString(b.Text, nil, 10);
        
        b:SetScript("OnClick", OnSendRequestClicked);
        b:SetScript("OnUpdate", SendRequestButton_Update);

        return prev, b;
    end,

    -- note
    function(b, prev, next)
        local f = CreateFrame("Button", "$parent_NoteHld", b);
        f:SetHeight(20);
        f:SetPoint("LEFT", prev, "RIGHT", 5, 0);
        f:SetPoint("RIGHT", next, "LEFT", -5, 0);

        fb:AddHoverBackground(f, {0,0,0,.5})
        fb:SetTooltip(f, LOC.DungeonNoteTooltip);
        local t = f:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");

        t:SetAllPoints();
        b.note = t;
        t:SetText("");

        f:SetScript("OnClick", function(self)  
            U:EditText(b.data.note, OnButtonEditNote, { b }, LOC.EditNote, LOC.Change) 
        end);

        return t;
    end,
}

function com.CreateButton(index, parent, prev)
    
    local f = CreateFrame("Button", "$parent_Btn" .. index, parent);

    if index == 1 then

        parent.firstButton = f;
        f:SetPoint("TOPLEFT", parent, "TOPLEFT", 0, -35);
        f:SetPoint("TOPRIGHT", parent, "TOPRIGHT", -32, -35);

    else

        f:SetPoint("TOPLEFT", prev, "BOTTOMLEFT");
        f:SetPoint("TOPRIGHT", prev, "BOTTOMRIGHT");

    end

    f:SetHeight(22);
    fb:SetupTextures(f, ButtonConfig, parent);
    
    Mixin(f, ButtonMixin);

    fb:ApplyFrameBuilders(f, ButtonBuilders);

    return f;
end