local A = select(2, ...);
local mod = A:Mod("lfg-frame");
--local M = A.Lfg;
local D = A.Api.Dungeon;
local U = A.Utils;
local dungeons = {}
local fb = A.Lib.FB;
local C = A.Constants;
local P = A:Mod("player"); --A.Player;
local R = A:Mod("request"); --A.Request;
local L = A.Localize.translate;

local Bus = A:Mod("chat-bus"); -- = A.ChatBus;

local TypeApi = A.Api.Type;
local RoleApi = A.Api.Role;
local SEND_DELAY_IN_SECONDS = 20;
local Cache = A.Cache;
local LFGDB = A:Mod("lfg-db");




local LOC = {
    SelectDungeon = L("Select dungeon"),
    NotSpecified = L("Not Specified"),
    Empty = L("---"),
    Close = L("Close"),
    Role = L("Specify your role"),
    DefaultRole = L("Your role"),
    DefaultRoleTooltip = L("Your default role, this role will be used for all dungeons with unspecified role"),
    DefaultNote = L("Your note"),
    DefaultNoteTooltip = L("Your default note, this note will be used for all dungeons with unspecified note"),
    DungeonRoleTooltip = L("You may specify special role for this dungeon"),
    DungeonNoteTooltip = L("You may specify special note for this dungeon"),
    EditNote = L("Edit note"), 
    Change = L("Change"),
    Send = L("Send"),
    CheckedExplain = L("When checked will autoresponse to a player who send a message in an LFG channel about this dungeon")
}


local DungeonFilter = {
    _values = {},
    _changeListeners = {},

    Set = function(self, key, value)
        local old = self._values[key];
        self._values[key] = value;

        if old ~= value then
            self:TriggerChange();
        end
    end,

    Get = function(self, key)
        if key == nil then return self._values end
        return self._values[key];
    end,

    TriggerChange = function(self)

        if not self._triggerChange then
            self._triggerChange = U:Debounce(function() self:OnChange() end, .1)
        end
        self._triggerChange();
    end,

    OnChange = function(self)

        self.changed = true;
        for _i, cb in pairs(self._changeListeners) do

            cb(self._values);
        end
        self.changed = false;
    end,

    AddListener = function(self, cb)
        table.insert(self._changeListeners, cb);
    end,
}


local function OnPlayerEditNote(value, button)
    P.note = value;
    Cache:SetOption("lfg.player.note", value);
    button:Update();
end



local FrameMixin = {

    GetRequestText = function(self, data)

        local send = {
            type = C.Lfg,
            zone = data.name,
            mode = data.mode,
            role = data.role or P.role,
            note = data.note or P.note,
        }
        
        local text = R:ToString(send);
        return text;
    end,

    SendRequest = function(self, data)
        local text = self:GetRequestText(data);
        Bus:SendChatMessage(text);
    end,

    GetData = function(self)
        if not self.data or DungeonFilter.changed then 
            DungeonFilter.changed = false;
            self.data = LFGDB:Get(DungeonFilter:Get())
        end
        --U:Log("#-", #self.data);
        return self.data;
    end,


    GetDungeonByRequest = function(self, request)
        local data = self:GetData();
        for _, model in pairs(data) do
            if model.checked and model.name == request.zone and model.mode == request.mode then
                return model;
                --print('>>', request.zone, request.mode);
            end
        end
    end,

    IsMyRoleSuitable = function(self, request, dungeon)
        local myrole = dungeon.role or P.role or nil;
        local iAmDd = myrole == C.Roles.mdd or C.Roles.rdd == myrole;


        if request.roles == nil or myrole == nil then
            return true
        end

        for _, rr in pairs(request.roles) do
            if rr == myrole then return true end
            if rr == C.Roles.dd and iAmDd then return true end
        end

        return false;

    end
}

local FauxFrameMixin = {
    GetData = function(self)
        return self:GetParent():GetData();
    end,
}

local FrameBuilders = {

    function(f)
        f.frameName = "LfgFrame";
        f:Hide();
        Mixin(f, FrameMixin);
    end,

    function(f)
        ---- add faux scroll frames
        
        local buttonsCount = 17;
        local fauxHolder = CreateFrame("Frame", "$parent_Holder", f);
    
        fauxHolder:SetPoint("TOPLEFT", f, "TOPLEFT", 8, -63);
        fauxHolder:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -3, 28);
        f.faux = fauxHolder;
    
        U:AddFauxScroll(fauxHolder, buttonsCount, A:Com("lfg-faux-button").CreateButton, FauxFrameMixin);

        ---- setup update buttons handlers
        
        local updateButtons = function() 
            fauxHolder:UpdateButtons(); 
        end
        f:SetScript("OnShow", updateButtons);
        DungeonFilter:AddListener(updateButtons);

    end,

    -- default role control
    function(f)

        

        --local fnt = h:GetFont();

        local r = CreateFrame("Button", "$parent_DefaultRoleButton", f);
        fb:AddHoverBackground(r, {0,0,0,.5});
        fb:SetTooltip(r, LOC.DefaultRoleTooltip);

        r:SetPoint("TOPLEFT", f, "TOPLEFT", 65, -37);
        r:SetSize(40,20);
        local t = r:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        --U:UpdateFontString(t, nil, 7);
        t:SetAllPoints();

        t:SetTextColor(1,1,1);

        r.Update = function(self) 
            t:SetText(P.role or "---");
        end
        r:Update();
        --t:SetText(P.role or "---");

        local click = function(self, arg1, arg2, checked)
            local v = self.value ~= "" and self.value or nil;
            P.role = v;
            Cache:SetOption("lfg.player.role", v);
            r:Update();
        end

        local isChecked = function(role)
            return P.role == role;
        end


        local dropdown = CreateFrame("Frame", "$parent_RoleDropdown", r, "UIDropDownMenuTemplate");
        local openDropdown = function(self) 
            PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
            local menuList = mod:GetRolesDropdownMenu(self, click, isChecked);
            U:EasyMenu(menuList, dropdown, r, 0, 0, displayMode);
        end
        r:SetScript("OnClick", openDropdown);


        local h = f:CreateFontString("$parent_DefaultRoleHeader", "OVERLAY", "GameFontNormalSmall");
        h:SetPoint("BOTTOM", r, "TOP", 0, 2);
        h:SetText(LOC.DefaultRole);

        return r;
    end,

    -- default note control
    function(f, prev)
        
        local r = CreateFrame("Button", "$parent_DefaultNoteButton", f);
        fb:AddHoverBackground(r, {0,0,0,.5});
        fb:SetTooltip(r, LOC.DefaultNoteTooltip);

        r:SetPoint("LEFT", prev, "RIGHT", 20, 0);
        r:SetSize(160,20);
        

        local t = r:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        --U:UpdateFontString(t, nil, 7);
        t:SetAllPoints();
        t:SetTextColor(1,1,1);

        r.Update = function(self) 
            t:SetText(P.note or "---");
        end
        r:Update();

        local h = f:CreateFontString("$parent_DefaultRoleHeader", "OVERLAY", "GameFontNormalSmall");
        h:SetPoint("BOTTOM", r, "TOP", 0, 2);
        h:SetText(LOC.DefaultNote);
        --h:SetJustifyH("LEFT");
        --U:UpdateFontString(h, nil, 7);

        r:SetScript("OnClick", function(self)  
            U:EditText(P.note, OnPlayerEditNote, { self }, LOC.EditNote, LOC.Change) 
        end);

    end,

    -- dungeons name filter control
    function(f)
        local anchor = f.faux.Buttons[1].zone;
        --print('--', anchor);
        local e = CreateFrame("EditBox", "$parent_Filters_Name", f, "InputBoxInstructionsTemplate");
        e:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT", 5, 10);
        e:SetPoint("BOTTOMRIGHT", anchor, "TOPRIGHT", -5, 10);
        --e:SetHeight(25)
        --name:SetSize(123, 25);
        local nameHght = 20;
        e:SetHeight(nameHght);
        e.Middle:SetHeight(nameHght);
        e.Left:SetHeight(nameHght);
        e.Right:SetHeight(nameHght);
        e:SetAutoFocus(false);
        e:SetText("");
        U:UpdateFontString(e, nil, 8);
        e:SetScript("OnKeyUp", U:Debounce(function() 
            local text = e:GetText();
            if text ~= nil then
                text = string.lower(text);
            end
            if text == "" then text = nil end
            DungeonFilter:Set("name", text);
        end, .5));
        -- e:SetScript("OnEnterPressed", function() f:OnApply() end);
        -- f.edit = e;

        -- e:SetScript("OnEscapePressed", function() f:OnEsc() end);
        
    end
}


function mod.CreateFrame(parent, id)

    local frame = CreateFrame("Frame", "$parent_Tab_LfgFrame", parent);

    fb:ApplyFrameBuilders(frame, FrameBuilders);

    return frame;

end

function mod:GetTabInfo()
    return {
        text = L("Lfg"),
        frameHeader = L("Looking for group"),
        tabFrameBuilder = mod.CreateFrame
    }
end