local A = select(2, ...);
local mod = A:Mod("lfm-frame");
--local M = A.Lfm;
local fb = A.Lib.FB;
local C = A.Constants;
local D = A.Api.Dungeon;
local L = A.Localize.translate;
local U = A.Utils;


local DATA = A:Mod("chat-data");
local REQ = A:Mod("lfm-request");




local LOC = {
    SelectDungeon = L("Select dungeon"),
    NotSpecified = L("Not Specified"),
    Close = L("Close"),
    Dungeons = L("Dungeons"),
    Mode = L("Mode"),
    Era = L("Addon"),
    Send = L("Send"),
    Invite = L("Invite"),
    Whisper = L("Whisper"),
    ClearZone = L("Unselect dungeon"),
    DungeonNotSelected = L("Dungeon not selected"),
    DungeonNote = L("Note"),
    EditNote = L("Edit dungeon note"),
    Change = L("Change"),
    Roles = L("Roles"),
    RolesTooltip = L("Specify roles you are looking for selected dungeon"),
    NoteTooltip = L("Specify additional information for selected dungeon"),
    SendButton_NotSpecifiedDungeon = L("Please select Dungeon you are looking for for sending a message in LFG channel"),
}


local function DungeonToMenuItem(dun, click, isChecked)
    local info = {
        text = dun.name,
        value = dun.id,
        func = click,
        checked = isChecked(dun.id),
    };
    return info;
end

local function GetDungeonsMenu(filters, click, isChecked)
    local dungeons = D:GetAll(filters)
    local result = {};
    for _, dungeon in pairs(dungeons) do
        table.insert(result, DungeonToMenuItem(dungeon, click, isChecked))
    end
    return result;
end

local function GetZoneMenuList(click, isChecked)
    
    local menuList = {
        { text = LOC.ClearZone, value = "", func = click, checked = isChecked(nil) },
        { separator = true },
        { text = LOC.SelectDungeon, isTitle = true, notCheckable = true  },
        { 
            text = L("Classic dungeons"), hasArrow = true, 
            menuList = GetDungeonsMenu({ era = C.Eras.Classic, raid = false }, click, isChecked)
        },
        { 
            text = L("Classic raids"), hasArrow = true, 
            menuList = GetDungeonsMenu({ era = C.Eras.Classic, raid = true }, click, isChecked)
        },
        {
            text = L("BC normal dungeons"), hasArrow = true,
            menuList = GetDungeonsMenu({ era = C.Eras.BC, heroic = false, raid = false }, click, isChecked)
        },
        {
            text = L("BC heroic dungeons"), hasArrow = true,
            menuList = GetDungeonsMenu({ era = C.Eras.BC, heroic = true, raid = false }, click, isChecked)
        },
        {
            text = L("BC raids"), hasArrow = true,
            menuList = GetDungeonsMenu({ era = C.Eras.BC, raid = true }, click, isChecked)
        },
        { separator = true },
        { text = LOC.Close, notCheckable = true },
    }
    return menuList;
end





local FrameMixin = {
    ShowSelectDungeonTooltip = function(self, frame)
        GameTooltip:SetOwner(frame);
        GameTooltip:AddLine(LOC.SendButton_NotSpecifiedDungeon, nil, nil, nil, true);
        GameTooltip:SetWidth(200);
        GameTooltip:Show();
    end,
    OnShow = function(self) 
        --REQ.weakFilter.myWishList = A:Mod("lfg-db"):GetCheckedHash();
        self.faux:UpdateButtons();
    end,
    OnHide = function(self) 
        --DATA.OnAdd = nil;
    end,
}

local FrameBuilders = {

    -- init
    function(f)
        f:Hide();
        f.tabId = tabId;
        f.frameName = "LfmFrame";
        
        Mixin(f, FrameMixin);
    
    end,

    -- faux
    function(f)
        local fauxHolder = CreateFrame("Frame", "$parent_Holder", f);
        f.faux = fauxHolder;
        fauxHolder:SetPoint("TOPLEFT", f, "TOPLEFT", 8, -63);
        fauxHolder:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -3, 28);
        local buttonsCount = 15;
    
        U:AddFauxScroll(fauxHolder, buttonsCount, A:Com("chat-faux-button").CreateButton, {
    
            GetData = function(self)
                local d = DATA:Get(function(entry) 
                   return entry.dungeon and REQ:IsZoneIdEqual(entry.dungeon.id) or false
                end);
                return d;
            end,
    
            UpdateButtonByItem = function(self, item)
                if not item or not item.__button_index then return end;
                local btn = self:GetButton(item.__button_index);
                if not btn or btn.data ~= item then return end;
    
                btn:Update(item);
    
            end,
        });
    end,

    -- scripts and handlers
    function(f)
        f:SetScript("OnShow", f.OnShow);
        f:SetScript("OnHide", f.OnHide);
    
        local ChangeHandler = function(event, player) 
            if not f:IsShown() then return end        
            f.faux:UpdateButtons();
        end
    
        DATA:On("player:update", ChangeHandler, f);
        REQ:On("filter:change", ChangeHandler, f);    
    end,

    -- select dungeon control
    function(f)



        --local fnt = h:GetFont();

        local r = CreateFrame("Button", "$parent_ZoneButton", f);
        fb:AddHoverBackground(r, {0,0,0,.5});
        fb:SetTooltip(r, function(self) f:ShowSelectDungeonTooltip(self) end);
        r:SetPoint("TOPLEFT", f, "TOPLEFT", 55, -37);
        r:SetSize(200, 20);

        local t = r:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        --U:UpdateFontString(t, nil, 7);
        t:SetPoint("LEFT", r, "LEFT", 20, 0);
        t:SetPoint("RIGHT", r, "RIGHT", 0, 0);
        t:SetJustifyH("LEFT");
        t:SetTextColor(1,1,1);

        r.Update = function(self) 
            local text = LOC.DungeonNotSelected;
            local zone = REQ:GetZone();
            if zone then 
                text = zone:ToString();
            end;
            t:SetText(text)
        end
        r:Update();

        f.zone = r;


        local h = f:CreateFontString("$parent_DefaultRoleHeader", "OVERLAY", "GameFontNormalSmall");
        h:SetPoint("BOTTOMLEFT", r, "TOPLEFT", 10, 2);
        h:SetText(LOC.SelectDungeon);
        h:SetJustifyH("LEFT");
        --U:UpdateFontString(h, nil, 7);

        local arrowBtn = U:CreateArrowButton("$parent_ArrowButton", r, { "LEFT", r, "LEFT", 0, 0 }, 16)

        local autoHideDelay, x, y, displayMode = 0, 20, 20, "MENU";
        local dropdown = CreateFrame("Frame", "$parent_ZoneDropdown", r, "UIDropDownMenuTemplate");

        local click = function(self) 
            local value = self.value;
            if value == "" then value = nil end
            REQ:SetZoneId(value);
            --DATA:SetDungeon(self.value);
            f.faux:UpdateButtons();
            r:Update();
            CloseDropDownMenus();
        end

        local isChecked = function(id) 
            return REQ:IsZoneIdEqual(id); 
        end

        local openDropdown = function() 
            PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
            local menuList = GetZoneMenuList(click, isChecked);
            U:EasyMenu(menuList, dropdown, r, 0, 0, displayMode);
        end

        r:SetScript("OnClick", openDropdown);
        arrowBtn:SetScript("OnClick", openDropdown);

        return r;
    end,

    -- dungeon edit role
    function(f, prev)

        local r = CreateFrame("Button", "$parent_RoleButton", f);
        fb:AddHoverBackground(r, {0,0,0,.5});
        fb:SetTooltip(r, LOC.RolesTooltip);

        r:SetPoint("LEFT", prev, "RIGHT", 10, 0);
        r:SetSize(120,20);

        local t = r:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        --U:UpdateFontString(t, nil, 7);
        t:SetAllPoints();

        t:SetTextColor(1,1,1);

        r.Update = function(self) 
            t:SetText(REQ:GetRolesText(" --- "));
        end
        r:Update();

        local click = function(self, arg1, arg2, checked)
            REQ:SetRole(self.value, checked);
            r:Update();
        end

        local isChecked = function(role)
            return REQ:HasRole(role);
        end


        local dropdown = CreateFrame("Frame", "$parent_RolesDropdown", r, "UIDropDownMenuTemplate");
        local h = f:CreateFontString("$parent_RolesHeader", "OVERLAY", "GameFontNormalSmall");
        h:SetPoint("BOTTOM", r, "TOP", 0, 2);
        h:SetText(LOC.Roles);

        --U:UpdateFontString(h, nil, 7);        


        local openDropdown = function() 
            PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);

            local menuList = {
                { text = LOC.Roles, isTitle = true, notCheckable = true  }
            };

            for _i, role in pairs(C.Roles) do
                table.insert(menuList, {
                    text = role,
                    value = role,
                    checked = isChecked(role),
                    func = click,
                    keepShownOnClick = true
                });        
            end


            table.insert(menuList, { separator = true });
            table.insert(menuList, { text = LOC.Close, notCheckable = true });

            U:EasyMenu(menuList, dropdown, r, 0, 0, "MENU");
        end

        r:SetScript("OnClick", openDropdown);

        return r;

    end,
    
    -- SendRequest button
    function(f, prev)
        local lfgb = CreateFrame("Button", "$parent_SendRequest", f, "UIPanelButtonTemplate");

        lfgb.enabledText = LOC.Send;
        lfgb:SetText(lfgb.enabledText);
        lfgb:SetScript("OnClick", function(self) 
            REQ:SendRequest();
        end);
        lfgb:SetPoint("TOPRIGHT", f, "TOPRIGHT", -10, -35);
        lfgb:SetWidth(70);

        
        lfgb:SetScript("OnUpdate", function(self) 
            local remains = REQ:GetSecondsToNewRequest();
            if remains > 0 then
                self:SetText(floor(remains));
                self:SetEnabled(false);
                return;
            else
                self:SetText(lfgb.enabledText);
            end
            local hasId = REQ:GetZoneId() ~= nil;
            self:SetEnabled(hasId);
        end);
        fb:SetTooltip(lfgb, function(self) 
            local req = REQ;
            local hasId = req:GetZoneId() ~= nil;
            if hasId then
                local txt = req:GetRequestText();
                U:WillBeSentTooltip(self, txt);
            else
                f:ShowSelectDungeonTooltip(self);
            end
        end);
        f.send = lfgb;
        return prev, lfgb;
    end,

    -- dungeon edit note
    function(f, prev, next)
        
        local r = CreateFrame("Button", "$parent_DefaultNoteButton", f);
        fb:AddHoverBackground(r, {0,0,0,.5});
        fb:SetTooltip(r, LOC.NoteTooltip);
        r:SetPoint("LEFT", prev, "RIGHT", 10, 0);
        r:SetPoint("RIGHT", next, "LEFT", -10, 0);
        r:SetHeight(20);
        --r:SetSize(120,20);
        

        local t = r:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        --U:UpdateFontString(t, nil, 7);
        t:SetAllPoints();
        t:SetTextColor(1,1,1);
        f.note = r;
        r.Update = function(self) 
            t:SetText(REQ:GetNote("---"));
        end
        r:Update();

        local h = f:CreateFontString("$parent_DefaultRoleHeader", "OVERLAY", "GameFontNormalSmall");
        h:SetPoint("BOTTOM", r, "TOP", 0, 2);
        h:SetText(LOC.DungeonNote);
        --h:SetJustifyH("LEFT");
        --U:UpdateFontString(h, nil, 7);

        r:SetScript("OnClick", function(self)  
            local req = REQ;
            U:EditText(req:GetNote(), function(value) req:SetNote(value); r:Update() end, nil, LOC.EditNote, LOC.Change) 
        end);

        return r;
    end,






}



function mod.CreateFrame(parent, id)

    local frame = CreateFrame("Frame", "$parent_Tab_LfmFrame", parent);

    fb:ApplyFrameBuilders(frame, FrameBuilders);

    return frame;

end

function mod:GetTabInfo()
    return {
        text = L("Lfm"),
        frameHeader = L("Looking for a member"),
        tabFrameBuilder = mod.CreateFrame
    }
end