local ANAME, A = select(1, ...);
local fb = A.Lib.FB;
local C = A.Constants;
local Cache = A.Cache;
local U = A.Utils;
local L = A.Localize.translate;


local TabsToBuild = {

    A:Mod("chat-frame"):GetTabInfo(),
    A:Mod("lfg-frame"):GetTabInfo(),
    A:Mod("lfm-frame"):GetTabInfo(),
    -- {
    --     text = "Lfg",
    --     frameHeader = "Looking for a group",
    --     tabFrameBuilder = function(p, id) return A.Lfg:BuildFrame(p, id) end
    -- },
    -- {
    --     text = "Lfm",
    --     frameHeader = "Looking for a member",
    --     tabFrameBuilder = function(p, id) return A.Lfm:BuildFrame(p, id) end
    -- },
}

local BLD = {};

local TabButtonMixin = {
    OnClick = function(self)
        PanelTemplates_Tab_OnClick(self, self:GetParent());
        PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
        --self:GetParent():ShowTab(self:GetID());
    end,
    OnDisable = function(self) 
        self.tabFrame:Show();
    end,
    OnEnable = function(self) 
        self.tabFrame:Hide();
    end
}




----------------------------------
----------------------------------

-- coommon builders for tab frame
local CommonTabBuilders = {

    -- positioning
    function(f)
        local parent = f:GetParent();
        f:SetPoint("TOPLEFT", parent, "TOPLEFT");
        f:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT");
    end,

    -- tab frame header
    function(f)
        local t = f:CreateFontString("$parent_Header", "OVERLAY", "GameFontNormal");
        t:SetPoint("TOPLEFT", f, "TOPLEFT", 70, 3);
        t:SetText(f.frameHeader);
        t:SetSize(300, 30);
        t:SetJustifyH("LEFT");
    end,

    -- send request button
    -- function(f)

    --     local lfgb = CreateFrame("Button", "$parent_SendRequest", f, "UIPanelButtonTemplate");
    --     lfgb:SetText(C.SendRequest);
    --     lfgb:SetScript("OnClick", function() 
    --         SendChatMessage(P:ToRequestMessage() ,"Say");
    --     end);
    --     lfgb:SetPoint("BOTTOM", f, "BOTTOM", 0, 4);
    --     lfgb:SetWidth(200);
    
    -- end
}



function BLD:CreateTabButton(frame, options)

    local id = #frame.Tabs + 1;

    local prev = frame.Tabs[id - 1];


    local t = CreateFrame("Button", "$parentTab" .. id, frame, "CharacterFrameTabButtonTemplate");
    
    t:SetID(id);
    t:SetText(options.text);

    if prev then
        t:SetPoint("LEFT", prev, "RIGHT", 0, 0);
    else
        t:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 10, 0);
    end
    Mixin(t, TabButtonMixin);

    t:SetScript("OnClick", t.OnClick);
    t:SetScript("OnDisable",t.OnDisable);
    t:SetScript("OnEnable", t.OnEnable);

    table.insert(frame.Tabs, t);

    return t, id;

end

function BLD:CreateTabFrame(parent, btnFrame, options)
    local id = btnFrame:GetID();
    local tab = options.tabFrameBuilder(parent, id);
    tab:SetID(id);
    tab.frameHeader = options.frameHeader or "";
    fb:ApplyFrameBuilders(tab, CommonTabBuilders);

    btnFrame.tabFrame = tab;

    return tab;
end

function BLD:CreateTab(frame, tabOptions)

    local btnFrame, id = self:CreateTabButton(frame, tabOptions);

    local tabFrame = self:CreateTabFrame(frame, btnFrame, tabOptions)

    PanelTemplates_SetNumTabs(frame, id);

    return tabFrame, id, tabButton;

end

----------------------------------
----------------------------------

local MainFrameMixin = {
    Tabs = {},
    --tabFrames = {},

    -- ShowTab = function(self, id)
    --     for _, tab in pairs(self.tabFrames) do
    --         if tab.tabId == id then
    --             tab:Show();
    --         else
    --             tab:Hide();
    --         end
    --     end
    --     PanelTemplates_SetTab(self, id);
    -- end,

    RegisterTab = function(self, options)
        return BLD:CreateTab(self, options);
    end
}


local Builders = {

    -- main things
    function(f)
        f:SetSize(800, 500);
        f:SetPoint("CENTER");
        fb:MakeMovable(f);
        local texture = _G[f:GetName().."Portrait"];
        texture:SetTexture(A.MainDataBroker.icon); --(LFG_EYE_TEXTURES.default); --("")
        --SetPortraitTexture(texture, "player");
    end,

    function(f)
        for _, tabOptions in pairs(TabsToBuild) do
            f:RegisterTab(tabOptions);
            --BLD:CreateTab(f, tabOptions);
        end
        PanelTemplates_SetTab(f, 1);
    end,

    -- creating lfm and lfg tabs
    -- function(f)
    --     f:CreateTab(1, function(p, id) return A.Lfg:BuildFrame(p, id) end);
    --     f:CreateTab(2, function(p, id) return A.Lfm:BuildFrame(p, id) end);
    -- end,

    -- create tab buttons
    -- function(f)
    --     local prev = f:CreateTabButton(1, C.Lfg, {"TOPLEFT", f, "BOTTOMLEFT", 10, 0})
    --     f:CreateTabButton(2, C.Lfm, {"LEFT", prev, "RIGHT", 0, 0});
    --     PanelTemplates_SetNumTabs(f, 2);
    --     f:ShowTab(1);
    -- end,

    -- Version string
    function(f)
        local t = f:CreateFontString("$parent_Version", "OVERLAY", "GameFontNormalSmall");
        t:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -12, 0);
        t:SetText(U:InlineColor("v" .. A.BUILD.appVersion, "ffffd0"));
        t:SetSize(60, 30);
        t:SetJustifyH("RIGHT");
    end,


    -- Config button
    function(f)
        local b = CreateFrame("Button", "$parent_ShowOptions", f, "UIPanelButtonTemplate");

        b:SetText(L("Configuration"));
        b:SetScript("OnClick", function(self) 
            A:Mod("addon-config"):Show();
        end);
        b:SetPoint("BOTTOM", f, "BOTTOM", 0, 4);
        b:SetWidth(110);
    end,

}

function A:CreateMainFrame()

    local f = CreateFrame("Frame", "TabuLfgFrame", UIParent, "ButtonFrameTemplate"); --"UIPanelDialogTemplate"); -- "UIPanelDialogTemplate");
    f:Hide();
    Mixin(f, MainFrameMixin);

    fb:ApplyFrameBuilders(f, Builders);

    return f;
end

function A:GetMainFrame()
    local f = A.mainFrame;
    if not f then
        f = self:CreateMainFrame();
        A.mainFrame = f;
    end
    return f;
end

function A:ShowMainFrame()
    local f = self:GetMainFrame();
    f:Show();
end

function A:HideMainFrame()
    local f = self:GetMainFrame();
    f:Hide();
end

function A:ToggleMainFrame()
    local f = self:GetMainFrame();
    if f:IsShown() then
        f:Hide();
    else
        f:Show();
    end
end