local A = select(2, ...);
local P = A:Mod("player"); --A.Player;
local C = A.Constants;
local U = A.Utils;
--local PDB = A.PlayersDB;
local MSG = A:Mod("message"); --A.Message;
local TypeApi = A.Api.Type;
local Cache = A.Cache;

local Bus = A:Mod("chat-bus");

local busFrame = CreateFrame("Frame");
--local Bus = CreateFrame("Frame");
--A.ChatBus = Bus;

Bus.LFG_Prefix = A.CommunicationId;

Bus.TotalPinged = 0;
Bus.PingResponses = 0;

Bus.TypeHandlers = {};
Bus.AllHandlers = {};

Bus.MinuteStarts = GetTime();
Bus.TotalMessages = 0;
Bus.MinuteMessages = 0;
Bus.MessagesPerMinute = 0;
Bus.ThrottledMessagesPerMinute = 0;
Bus.TotalThrottledMessages = 0;
Bus.ThrottledMinuteMessages = 0;
-- should be filled
Bus.playerPostfix = nil;

local ChatThrottler;


Bus.verbose = true;

----------------------------------------
------UTILS-----------------------------

local function normalizePlayerName(playerName)

    local index = string.find(playerName, "-");

    if index and not Bus.playerPostfix then
        local pstfx = U:TextSplit(playerName, "-");
        Bus.playerPostfix = U:JoinToString(pstfx, "-", 2);
        return playerName;
    elseif index then
        return playerName;
    elseif not Bus.playerPostfix then
        -- very rare case
        return;
    end

    -- here we go if there is no index and postfix detected

    return playerName .. "-" .. Bus.playerPostfix;

end

local function normalizeGuid(guid)
    if type(guid) ~= "string" or guid == "" then
        return;
    end
    
    local prefix = string.lower(strsub(guid, 1, 7));
    if prefix == "player-" then
        return guid;
    else
        --print(" # ?? ", guid, prefix);
    end
end

function Bus:GetLfgChannelId()
    local id = U:GetLFGChannelName();
    return id;
end

function Bus:GetGeneralChannelId()
    local id = U:GetGeneralChannelName();
    return id;
end

function Bus:IncrementCounter()
    self.TotalMessages = self.TotalMessages + 1;
    self.MinuteMessages = self.MinuteMessages + 1;
end

function Bus:IncrementThrottledCounter()
    self.TotalThrottledMessages = self.TotalThrottledMessages + 1;
    self.ThrottledMinuteMessages = self.ThrottledMinuteMessages + 1;
end

----------------------------------------
------SENDING---------------------------

local function SendToLfgChat(text)

    if not Cache:GetOption("chat.send.toLfg") then
        return;
    end

    local lfgChannelId = Bus:GetLfgChannelId();
    if not lfgChannelId then
        U:Msg(L("LookingForGroup channel not found. It seems you are not joined"));
        return;
    end


    if Cache:GetOption("chat.send.LfgSpread") then
        local howMany = floor(Bus.MessagesPerMinute / 75) + 1;
        for _i=1, howMany do
            SendChatMessage(text, "CHANNEL", nil, lfgChannelId);
        end
    else
        SendChatMessage(text, "CHANNEL", nil, lfgChannelId);
    end
    return true;
end

local function SendToGeneral(text)

    if not Cache:GetOption("chat.send.toGeneral") then
        return;
    end

    local generalChannelId = Bus:GetGeneralChannelId();
    if generalChannelId then
        SendChatMessage(text, "CHANNEL", nil, generalChannelId);
        return true;
    else
        U:Log("no general channel")
    end
end

function Bus:SendChatMessage(text)
    local sendedLfg = SendToLfgChat(text);
    local sendedGeneral = SendToGeneral(text);
    local wasSent = sendedLfg or sendedGeneral or false;

    U:Msg(U:InlineColor("was " .. (wasSent and "" or "not ") .."sent: ", "00aaff"), U:InlineColor(text, "ffdd00"));
    if not wasSent then
        SendChatMessage(text, "SAY");
    end

end

function Bus:SendAddonMessage(text, player)
    local playerName;
    local playerType = type(player);
    if playerType == "table" then
        playerName = player.name
    elseif playerType == "string" then
        playerName = player
    end

    if not playerName then
        U:Log("player name was not provided for sending addon message");
        return;
    end

    local ret = C_ChatInfo.SendAddonMessage(Bus.LFG_Prefix, text, "WHISPER", playerName);
    if not ret then
        U:Msg('send internal message to ' .. playerName .. " fails");
    end
end

function Bus:SendPing(playerName)
    local text = MSG:BuildPingRequest(playerName);
    self:SendAddonMessage(text, playerName);
    Bus.TotalPinged = Bus.TotalPinged + 1;
    --U:Log("> ping", playerName);
end

function Bus:SendPingResponse(playerName)
    local text = P:BuildPingResponse(playerName);
    self:SendAddonMessage(text, playerName);
end




----------------------------------------
------HANDLERS--------------------------


function Bus:OnChatMsgAddon(comid, text, channel, playerName, playerName2)
    if not self.started then return end;

    playerName = normalizePlayerName(playerName);
    if not playerName then return end
    
    local message = MSG:CreateAddon(comid, text, channel, playerName, playerName2);
    
    if not message then 
        return 
    end;

    self:Trigger("addon:message", message);

end


function Bus:OnChatMsgChannel(text, playerName, languageName, channelName, playerName2, specialFlags, zoneChannelID, channelIndex, channelBaseName, unused, lineID, guid, bnSenderID, isMobile, isSubtitle, hideSenderInLetterbox, supressRaidIcons)
    -- drop message if we are not started
    if not self.started then return end;

    -- all messages should be counted for stats
    self:IncrementCounter();

    -- system and my own messages should be dropped
    if playerName == nil or playerName == "" or P:IsMyGuid(guid) then return end
    
    playerName = normalizePlayerName(playerName);
    if not playerName then
        U:Log("OnChatMsg:", "no playerName", playerName);
        return;
    end
    
    -- drop message if exactly this message was posted in near 10 seconds
    if not ChatThrottler:TryAdd(playerName, true) then 
        --U:Log("OnChatMsg:", "throttled", playerName);
        return;
    end

    self:IncrementThrottledCounter();

    guid = normalizeGuid(guid);
    local PDB = A:Mod("players-db");
    -- populating player info, without player level because this information is not available
    local player = PDB:GetOrCreate(playerName, "chat", { id = guid, name = playerName2, fullName = playerName });

    
    -- -- populating message object
    local message = MSG:Create(text, playerName, player);
    if not message then 
        --U:Log("OnChatMsg:", "no message", playerName);
        return 
    end;

    -- trying to detect if it is a valid message
    message:TryDetect();

    self:Trigger("chat:message", message);

end

----------------------------------------
------INITIALIZE------------------------

function busFrame:OnEvent(event, ...)
    if event == "CHAT_MSG_ADDON" then
        Bus:OnChatMsgAddon(...);
    elseif event == "CHAT_MSG_CHANNEL" or event == "CHAT_MSG_SAY" then
        Bus:OnChatMsgChannel(...);
    -- elseif event == "PLAYER_ENTERING_WORLD" then
    --     local isLogin, isReload = ...;
    --     self:OnPlayerEnteringWorld(isLogin, isReload)
    end
end



function Bus:OnStart()

    if not self._messagePrefixRegistered then
        self._messagePrefixRegistered = true;
        C_ChatInfo.RegisterAddonMessagePrefix(self.LFG_Prefix);
    end

    local storeForInSeconds = function() return A.Cache:GetOption("chat.throttler.storeFore"); end
    local cleanEachInSeconds = function() return A.Cache:GetOption("chat.throttler.cleanIn"); end

    if not self._chat_throttler then
        self._chat_throttler = true;
        ChatThrottler = U:CreateThrottler(function(message) return message.time end, storeForInSeconds, cleanEachInSeconds, "chat-message");
    end

    if A.Cache:GetOption("chat.throttler") then
        ChatThrottler:Start();
    end

    --self.started = true;

    if not self._frame_events_registered then
        self._frame_events_registered = true;
        busFrame:RegisterEvent("CHAT_MSG_ADDON");
        busFrame:RegisterEvent("CHAT_MSG_CHANNEL");
        busFrame:RegisterEvent("CHAT_MSG_SAY");
    end
    busFrame:SetScript("OnEvent", busFrame.OnEvent);
    
    
    Bus._statsTimer = C_Timer.NewTicker(60, function()  
        Bus.MinuteStarts = GetTime();
        Bus.MessagesPerMinute = Bus.MinuteMessages;
        Bus.MinuteMessages = 0;
        Bus.ThrottledMessagesPerMinute = Bus.ThrottledMinuteMessages;
        Bus.ThrottledMinuteMessages = 0;
    end);
    


end

function Bus:OnStop()

    ChatThrottler:Stop();
    busFrame:SetScript("OnEvent", nil);
    Bus._statsTimer:Cancel();

end
