local A = select(2, ...);
local TypeApi = A.Api.Type;
local U = A.Utils;
local MSG = A:Mod("message"); --A.Message;
local mod = A:Mod("chat-data");
local REQ = function() return A:Mod("lfm-request") end

local Bus = A:Mod("chat-bus");
local PDB = function() return A:Mod("players-db") end
local SEND_DELAY_IN_SECONDS = 20;
local THIRTY_MINUTES = 60 * 30;
local OLD_WHEN = 60 * 5;
local LFGDB = function() return A:Mod("lfg-db") end
local RoleApi = A.Api.Role;

local emptyTable = {};

local function LfmUserSorter(a,b)
    local ad = a.dungeon;
    local bd = b.dungeon;
    local an = (a.player or {}).fullName;
    local bn = (b.player or {}).fullName;
    if ad == nil and bd == nil then
        return an < bn;
    elseif ad == nil and bd ~= nil then
        return false;
    elseif ad ~= nil and bd == nil then
        return true;
    elseif ad.id == bd.id then
        return an < bn;
    else
        return ad.id < bd.id
    end
end


local DATAMIXIN = {

    users = {}, 
    usersByName = {},

    GetUser = function(self, name)
        local users = self:GetUsersByName();
        return users[name];
    end,

    SetUser = function(self, user)
        local users = self:GetUsersByName();
        users[user.player.fullName] = user;
    end,

    RemoveUser = function(self, user)
        local users = self:GetUsersByName();
        users[user.player.fullName] = nil;        
    end,

    GetUsersByName = function(self)
        if not self.usersByName then
            self.usersByName = {}
        end
        return self.usersByName;
    end,

    GetOldAfterValue = function(self, entry)
        local detected = entry:IsDetected();

        local key = "chat.entry.isOldAfter"; --.dungeon.Lfm"

        if detected then
            local r_type = entry.type and entry.type.id or nil;
            if entry.dungeon then
                key = key .. ".dungeon";
                if r_type then
                    key = key .. "." .. r_type;
                end
            elseif entry.quest then
                key = key .. ".quest";
            end
        else
            key = key .. ".undetected";
        end

        return A.Cache:GetOption(key);

    end,

    EntryIsFreshEnough = function(self, entry, options)
        options = options or emptyTable;
        
        local now = options.now or GetTime();
        
        local oldAfter = self:GetOldAfterValue(entry);

        if oldAfter == nil or oldAfter == 0 then
            return true;
        end
        
        return now - entry.time < oldAfter;
    end,

    EntryIsMatch = function(self, entry, options)
        options = options or emptyTable;
        if not options.filter then return end
        local fliterType = type(options.filter);
        if fliterType == "function" then
            return options.filter(entry)
        elseif fliterType == "table" then
            return MSG:IsMatch(entry, options.filter);
        else
            return;
        end
        -- hasZone = hasZone or REQ():HasZone();
        -- if hasZone then
        --     local res = user.dungeon and REQ():IsZoneIdEqual(user.dungeon.id) or false;
        --     return res;
        -- else
        --     return MSG:IsMatch(user, REQ().weakFilter or {});
        -- end
    end,


    OnDataUpdate = function (self, res)
        self:Trigger("data:update", res);
    end,

    _Get_ProcessOne = function(self, entry, result, options)
        if not entry.id or options.keys[entry.id] then return end
        options.keys[entry.id] = true;


        if not self:EntryIsFreshEnough(entry, options) then return end
        if not self:EntryIsMatch(entry, options) then return end
        
        table.insert(result, entry);
        
    end,

    Get = function(self, filter)
        local res = {};
        local keys = {};

        local now = GetTime();
        --local hasZone = REQ():HasZone();

        local users = self:GetUsersByName();

        local options = {
            now = now,
            filter = filter,
            --hasZone = hasZone,
            keys = keys,
        }

        local index = 0;

        for _, user in pairs(users) do
            index = index + 1;
            options.index = index;
            self:_Get_ProcessOne(user, res, options);
        end

        table.sort(res, LfmUserSorter)
        self._lastEntries = res;
        self:OnDataUpdate(res);

        return res;
    end,

    AddPlayer = function(self, user)
        local id = user.id
        
        local entry = self:GetUser(id)

        if not entry then
            entry = user;
            self:SetUser(entry);
        else
            Mixin(entry, user)
        end
        --Tabu.dump(entry);
        self:OnPlayerUpdate();

        --print("< player updated >");

    end,


    OnPlayerUpdate = function(self, player)
        self:Trigger("player:update", res);
    end,

    SetDungeon = function(self, id)
        if self.currentId ~= id then
            self.currentId = id;
            self:CleanUp();
        end
    end,

    CleanUp = function(self)
        wipe(self.users); -- = {};
    end,

}
Mixin(mod, DATAMIXIN);


mod._removeOldEntriesTimer = C_Timer.NewTicker(60, function()  
    local users = mod:GetUsersByName();

    for key, user in pairs(users) do
        if not mod:EntryIsFreshEnough(user) then
            mod:RemoveUser(user);
        end
    end

end);


local function TrySendAutoResponse(message)
    if not message.dungeon then return end;

    local dun = LFGDB():GetById(message.dungeon.id);
    if not dun or not dun.checked then 
        return 
    end

    local P = A:Mod("player"); --A.Player;

    local myrole = dun.role or P.role;

    local rolesAreMatch = RoleApi:IsRolesHasRole(message.roles, myrole);

    if rolesAreMatch then
        local text = P:LfmResponse(dun);
        --U:Log("> sending auto response");
        Bus:SendAddonMessage(text, message.player);        
    end
end

local function ChatMessageHandler(event, message)
    if message.type and message.type.id == TypeApi.Ping.id then
        U:Log("chat PING >", message.sender);
    else
        -- U:Log("DATA >")
        mod:AddPlayer(message);

        if message.native and message.player.fullName and message.player.version then
            PDB():RememberAddonUser(message.player);
        else
            PDB():TrySendPing(message.player.fullName);
        end

        TrySendAutoResponse(message);

    end
end

local function AddonMessageHandler(event, message)
    if not message or not message.type then return end

    if message.type == TypeApi.Ping then        
        if message.pingRequest then
            PDB():RegisterPingRequest(message.player);
        elseif message.pingResponse then
            PDB():RegisterPingResponse(message.player);
        end
        return;
    end

    if message.type == TypeApi.Lfg or message.type == TypeApi.Lfm then
        mod:AddPlayer(message);
        return;
    end

end

mod:ListenTo(Bus, "chat:message", ChatMessageHandler);
mod:ListenTo(Bus, "addon:message", AddonMessageHandler);