local A = select(2, ...);
local DungeonApi = A.Api.Dungeon;
local U = A.Utils;
local Request = A:Mod("request"); -- A.Request;
local mod = A:Mod("lfm-request")
local C = A.Constants;

local function Bus()
    return A:Mod("chat-bus")
end

local SEND_DELAY_IN_SECONDS = 20;

Mixin(mod, {

    values = {},
    weakFilter = {
        eras = {},
        modes = {},        
    },

    OnFilterChange = function(self)

        self:Trigger("filter:change");
    end,

    OnUpdate = function(self)
        self:Trigger("request:change");
        -- for _, handler in pairs(self.updateHandlers) do
        --     handler();
        -- end
    end,


    Set = function(self, arg1, arg2)
        local t = type(arg1);
        if t == "table" then
            if arg2 == true then
                wipe(self.values)
            end
            Mixin(self.values, arg1);
        elseif t == "string" then
            self.values[arg1] = arg2;
        end
        self:OnUpdate();
        --print(": set", arg1, arg2)
    end,

    Get = function(self, key)
        if key == nil then
            return self.values;
        else
            return self.values[key];
        end
    end,

    SetZoneId = function(self, id)
        self:Set("zoneId", id);
        local zone = id ~= nil and DungeonApi:GetOne(id) or nil;
        self:Set("zone", zone);
    end,

    GetZoneId = function(self, id)
        return self:Get("zoneId");
    end,

    GetZone = function(self)
        return self:Get("zone");
    end,

    HasZone = function(self)
        return self:GetZone() ~= nil;
    end,

    IsZoneIdEqual = function(self, id)
        return self:GetZoneId() == id;
    end,

    GetNote = function(self, ifEmpty)
        return self:Get("note") or ifEmpty;
    end,

    SetNote = function(self, value)
        self:Set("note", value);
    end,

    GetRolesText = function(self, ifEmpty)

        local roles = self:Get("roles");
        if not roles then
            return ifEmpty
        end

        local t = {};
        for role, roleValue in pairs(roles) do
            if roleValue == true then
                table.insert(t, role);
            end
        end
        local res = U:JoinToString(t, C.RequestRoleSeparator);
        if res == nil or res == "" then
            res = ifEmpty;
        end
        return res;
    end,

    SetRole = function(self, role, val)
        local roles = self:Get("roles") or {};
        roles[role] = (val == true);
        self:Set("roles", roles);
    end,

    HasRole = function(self, role)
        local roles = self:Get("roles");
        return roles ~= nil and roles[role] == true;
    end,

    RoleIsMatch = function(self, role)
        local roles = self:Get("roles");
        return roles == nil or roles[role] == true;
    end,
    -----------
    lastSendedRequest = GetTime()  - (SEND_DELAY_IN_SECONDS),

    UpdateLastSendedRequest = function(self)
        self.lastSendedRequest = GetTime();
    end,

    GetSecondsToNewRequest = function(self)
        local t = GetTime();
        local d = t - self.lastSendedRequest;
        return SEND_DELAY_IN_SECONDS - d;
    end,
    ------------------------

    SendRequest = function(self)

        self:UpdateLastSendedRequest();

        local text = self:GetRequestText();
        if text == nil then
            --print('Request was not sent, dungeon not found');
            return;
        end

        Bus():SendChatMessage(text);

    end,

    GetRequestText = function(self)
        local dun = self:GetZone();
        if dun == nil then return end;

        local request = {
            type = C.Lfm,
            zone = dun.name,
            mode = dun.mode,
            roles = self:Get("roles"),
            note = self:Get("note"),
        }
        --Tabu.dump(request);
        return Request:ToString(request);

        -- local t = {
        --     C.Lfm, 
        --     dun.name  .. C.RequestModeSeparator .. dun.mode,
        --     self:GetRolesText(),
        --     self:GetNote()
        -- }
        -- return U:JoinToString(t, C.RequestSeparator);
    end,

});


