local A = select(2, ...);

local C = A.Constants;
local Bus = A:Mod("chat-bus"); -- = A.ChatBus;
local MSG = A:Mod("message"); --A.Message;
local U = A.Utils;
local TypeApi = A.Api.Type;
local DungeonApi = A.Api.Dungeon;
local RoleApi = A.Api.Role;
local ModeApi = A.Api.Mode;

local PDB = function() return A:Mod("players-db") end;

------ Begin MessageMixin -----------

local function MessageMixin_TryParseNativeLfmMessage(self)

    local _t, _zone, _roles = unpack(self.words); --;select(2, unpack());
    local noteWords = { select(4, unpack(self.words)) };

    local dungeon = DungeonApi:TryGues(_zone);
    if not dungeon then
        return false;
    end
    

    local roles = RoleApi:TryParseRoles(_roles);
    if not roles and _roles then
        table.insert(noteWords, 1, _roles);
    end
    
    self.dungeon = dungeon;
    self.roles = roles;
    self.note = U:JoinToString(noteWords, " ");

    return true;
    --print("# -- ", _zone, _roles, note);
end

local function MessageMixin_TryParseNativeLfgMessage(self)
    local _t, _zone, _roles = unpack(self.words);
    local noteWords = { select(4, unpack(self.words)) };

    local dungeon = DungeonApi:TryGues(_zone);
    if not dungeon then
        return false;
    end
    

    local roles = RoleApi:TryParseRoles(_roles);
    if not roles and _roles then
        -- local noteAfter = note;
        -- note = _roles;
        -- if noteAfter then
        --     note = note .. noteAfter
        -- end
        table.insert(noteWords, 1, _roles);
    end

    self.dungeon = dungeon;
    if roles then
        self.role = roles[1];
    end
    self.note = U:JoinToString(noteWords, " ");

    return true;
end

local function MessageMixin_TryParseNativeMessage(self)
    if self.type == TypeApi.Lfm then
        return MessageMixin_TryParseNativeLfmMessage(self, mType);
    elseif self.type == TypeApi.Lfg then
        return MessageMixin_TryParseNativeLfgMessage(self, mType);
    else
        print('missing parse case for ', self.type.id);
    end
end

local QUEST_PATTERN = "(%[[^(]*%((%d*)%)%])";
-- /dump string.match("asdasdasd [[70]абевегедейка зеленая чпочка (23)] пакля равкла [Да здра перма(12345)]", "%[[^(]*%((%d*)%)%]");
-- /dump string.match("asdasdasd [[70]абевегедейка зеленая чпочка (23)] пакля равкла [Да здра перма(12345)]", "(%[[^(]*%((%d*)%)%])");
local MessageMixin = {}


local function DetectQuest(text)
    local questName, questId = string.match(text, QUEST_PATTERN);
    if not questId then return end
    local patternName = string.gsub(questName, "([%[%]%(%)])", "%%%1");
    local questText = string.gsub(text, patternName, '');
    local quest = {
        id = tonumber(questId),
        name = questName,
        patternName = patternName,
        text = questText
    }
    return quest;
end

local function DetectWord(msg, word, nextWord)

    if not msg.type then
        local type = TypeApi:GetByShort(word);
        if type then
            msg.type = type;
            return true;
        end
    end

    if not msg.dungeon then
        local dungeon, foundBy = DungeonApi:GetByShort(word, nextWord);
        if dungeon then
            msg.dungeon = dungeon;
            msg.dungeonFoundBy = foundBy;
            return true;
        end
    end

    local role = RoleApi:GetByShort(word);
    if role then
        if not msg.roles then
            msg.roles = {}
        end
        --msg.roles[role.id] = role;
        -- this leads to multiplying role
        table.insert(msg.roles, role);
        return true;
    end
    
    local mode = ModeApi:GetByShort(word);
    if mode then
        msg.mode = mode;
        return true;
    end

end

local function ProcessWord(index, words, msg)
    local word = words[index];

    if C.Shorts.IsIgnored(word) then 
        return 
    end

    local nextWord = words[index + 1];
    local detected = DetectWord(msg, word, nextWord);
    if not detected then
        local add = nil;
        local num = tonumber(word);
        if num ~= nil then
            add = num;
        elseif strlenutf8(word) >= 2 then
            add = word;
        end

        if add ~= nil then
            if msg.autoNote ~= "" then
                msg.autoNote = msg.autoNote .. " ";
            end
            msg.autoNote = msg.autoNote .. add;
        end
    end
end

function MessageMixin:TryDetect(isDebug)
    local looksLikeNative = false;
    if #self.words < 2 then
        self.valid = false;
        return;
    end

    local mType = TypeApi:GetByShort(self.words[1]);

    if mType ~= nil then
        self.type = mType;
        looksLikeNative = MessageMixin_TryParseNativeMessage(self);
        if looksLikeNative then            
            self.valid = true;
            --return;
        end
    else
        -- print("type not detected");
    end

    --U:Log("looks like native: ", looksLikeNative);
    
    self.autoNote = "";

    self.quest = DetectQuest(self.text);

    for i = 1, #self.normalizedWords do
        ProcessWord(i, self.normalizedWords, self);
    end

    local mode = self.mode or ModeApi.Normal;
    local foundBy, dng;

    if not self.dungeon and not self.quest then            
        dng, foundBy = DungeonApi:FindByShort(self.normalizedText, "dungeon");
        --U:Log("look up by short: ", foundBy);
        self.dungeon = dng;
        self.dungeonFoundBy = foundBy;
    end

    if self.dungeon then
        if self.dungeon.mode ~= mode.id then
            self.dungeon = DungeonApi:SwapMode(self.dungeon);
        end
    end


    if self.type and self.roles and #self.roles > 0 then
        if self.type.id == C.Lfg then
            self.role = self.roles[1];
        end
    end

    if self.autoNote == "" then
        self.autoNote = nil;
    end

end


function MessageMixin:IsDetected()
    if self.dungeon or self.quest then
        return true;
    else
        return false
    end
end


------ End MessageMixin -----------
--/run TabuLfgTest("Алькатрац об нид ТАНК и 1дд!!!!")
function MSG:ParseText(text, msg)
    text = text or "";
    local isDebug = msg == nil;

    local log = isDebug and print or function() end;

    msg = msg or {};
    local words_separator = " ";
    local words = msg.words;
    if not words then
        words = U:TextSplit(text, words_separator);
        msg.words = words;
    end
    --log("w:", words);

    local normalizedText = msg.normalizedText;
    if not normalizedText then
        normalizedText = string.gsub(text, '[%p%c]', words_separator);
        msg.normalizedText = normalizedText;
    end

    local normalizedWords = msg.normalizedWords;
    if not normalizedWords then
        normalizedWords = U:TextSplit(normalizedText, words_separator);
        msg.normalizedWords = normalizedWords;
    end

    --log("nw:", normalizedWords);

    for i = 1, #normalizedWords do

        local word = normalizedWords[i];
        local nextWord = normalizedWords[i+1];
        local detected = DetectWord(msg, word, nextWord);
        log("#", detected, word, nextWord);

    end

    local mode = msg.mode or ModeApi.Normal;

    if not msg.dungeon then
        dng, foundBy = DungeonApi:FindByShort(self.normalizedText);
        msg.dungeon = dng;
        msg.dungeonFoundBy = foundBy;        
    end

    if msg.dungeon then
        if msg.dungeon.mode ~= mode.id then
            msg.dungeon = DungeonApi:SwapMode(msg.dungeon);
        end
    end


    if msg.type and msg.roles and #msg.roles > 0 then
        if msg.type.id == C.Lfg then
            msg.role = msg.roles[1];
        end
    end

    if isDebug then
        Tabu.dump(msg)
    end
    return msg;
end

function TabuLfgTest(txt)
    MSG:ParseText(txt);
end

function MSG:Create(text, playerName, player, separator)
    separator = separator or " ";

    --U:Log("------------------");
    --U:Log("origi: ", text);
    text = text or "";
    text = string.gsub(text, "{[^}]*}", "");
    text = string.gsub(text, "%s+", " ");
    --U:Log("fixed: ", text);

    local words = U:TextSplit(text, separator);
    

    local text_formated = string.gsub(text, '[%p%c]', ' ');
    words_formated = U:TextSplit(text_formated, separator);
    
    local normalizedText = string.gsub(text_formated, '[%p%c%s]', '');

    local mes = {
        id = player.fullName,
        time = GetTime(),
        text = text,
        words = words,
        normalizedText = string.lower(normalizedText),
        normalizedWords = words_formated,
        player = player,
        sender = playerName
    }

    Mixin(mes, MessageMixin);

    return mes;
end


function MSG:ExtractPlayer(msg, chat_playerName, playerName2)
    if type(msg.guid) ~= "string" or msg.guid == "" then
        msg.guid = nil;
    end

    local fields = {
        id = msg.guid,
        name = msg.name or playerName2,
        class = msg.engClass or msg.cls or nil,
        level = msg.level or msg.lvl or nil,
        talents = msg.tals,
        avgItemLevel = msg.avgILevel or msg.ail or nil,
        version = msg.v,
        native = true,
    };
    
    local player = PDB():GetOrCreate(chat_playerName, "addon", fields, true);

    return player;
end

function MSG:CreateAddon(comid, text, channel, chat_playerName, playerName2)

    if comid ~= Bus.LFG_Prefix then return end

    --U:Log("AddonMsg", "[", comid, "]", chat_playerName, playerName2, text);

    local msg, size = U:AddonMessageToTable(text);

    if size == 0 or not msg.type then
        U:Log("no msg.type");
        return
    end

    local msgtype = msg.type and TypeApi:GetById(msg.type) or nil;
    
    if msgtype == nil then 
        U:Log("msg type not fouond for", msg.type);
        return 
    end

    local parsed;

    if msgtype == TypeApi.Ping then

        parsed = {
            id = chat_playerName,
            type = msgtype,
        }

        parsed.player = self:ExtractPlayer(msg, chat_playerName, playerName2);
        
        if msg.guid then
            parsed.pingResponse = 1;
            -- local player = self:ExtractPlayer(msg, chat_playerName, playerName2);
            -- parsed.player = player;
        else
            parsed.pingRequest = 1;
            -- parsed.sender = playerName2 or chat_playerName
        end

        Mixin(parsed, MessageMixin);

    elseif msgtype == TypeApi.Lfg then

        local player = self:ExtractPlayer(msg, chat_playerName, playerName2);

        local dungeon = DungeonApi:GetById(msg.dungeonId or msg.did);

        local roles = RoleApi:TryParseRoles(msg.role);

        local role;

        if msgtype == TypeApi.Lfg then
            if roles then
                role = roles[1];
            end
            roles = nil;
        end

        parsed = {
            id = player.fullName,
            type = msgtype,
            time = GetTime(),
            --text = text,
            words = words,
            dungeon = dungeon,
            player = player,
            note = msg.note,
            role = role,
            roles = roles,
            --avgILevel = msg.avgILevel or msg.ail,

            native = true,
            valid = true,
        }

        Mixin(parsed, MessageMixin);
        
    end
    parsed.text = "auto response";
    parsed.sender = chat_playerName;
    return parsed;
end

function MSG:CreateAddonOld(comid, text, channel, chat_playerName, playerName2)

    local words = U:TextSplit(text, C.RequestSeparator);

    local rtype, dungeonId, playerId, parsed_playerName, playerClass, playerSystemClass, level, _roles = unpack(words);
    local note = U:JoinToString(words, C.RequestSeparator, 9);

    local type = TypeApi:GetById(rtype);


    local player = PDB():GetOrCreate(chat_playerName, "addon", {
        id = playerId,
        name = parsed_playerName,
        class = playerSystemClass,
        level = level,

    }, true);
    
    local dungeon = DungeonApi:GetById(dungeonId);

    local roles = RoleApi:TryParseRoles(_roles);
    local role;
    if type == TypeApi.Lfg then
        if roles then
            role = roles[1];
        end
        roles = nil;
    end

    local mes = {
        id = player.id,
        type = type,
        time = GetTime(),
        --text = text,
        words = words,
        dungeon = dungeon,
        player = player,
        note = note,
        role = role,
        roles = roles,
        native = true,
        valid = true,
    }
    Mixin(mes, MessageMixin);
   return mes;
end

function MSG:BuildPingRequest()
    local tbl = {
        type = TypeApi.Ping.id,
        v = A.BUILD.appVersion
    }
    return U:TableToAddonMessage(tbl);
end


function MSG:IsMatch(msg, filters)

    if filters.eras and msg.dungeon then
        if filters.eras[msg.dungeon.era] == false then
            return false
        end
    end

    if filters.modes and msg.dungeon then
        if msg.dungeon.groupType == C.DungeonSizes.Group and filters.modes[msg.dungeon.mode] == false then
            return false
        end
    end


    if filters.showQuests == false and msg.quest then
        return false;
    end

    if filters.showUndetected == false and not msg:IsDetected() then
        return false;
    end

    if msg.dungeon and filters.inMyWishList == true and filters.myWishList ~= nil then
        if filters.myWishList[msg.dungeon.id] ~= true then
            return false;
        end
    end

    return true;
end


--- В СТАТИК КАРЫ НИД ЭЛЕМ, ШП/РДРУ. Без гильд обязательств, можете остаться в своей. Подробности в пм.