local A = select(2, ...);
local P = A:Mod("player"); --A.Player;
local C = A.Constants;
local R = A:Mod("request"); --A.Request;
local U = A.Utils;
local DungeonApi = A.Api.Dungeon;
local TypeApi = A.Api.Type;
--local DB = A.PlayersDB;
local Cache = A.Cache;

--P.mode = C.Modes.Normal;


function P:Set(key, value)
    self[key] = value;
end

function P:ToRequestMessage()
    return R:ToString(self);
end

function P:ZoneOn(id, silent)
    self.zone[id] = true;
    if silent then return end;
end

function P:ZoneOff(id, silent)
    self.zone[id] = nil;
    if silent then return end;
end

function P:HasZone(id)
    return self.zone[id] == true;
end

function P:ZoneClear(id, silent)
    for id, _ in pairs(self.zone) do
        self:ZoneOff(id, silent);
    end

    if silent then return end;

end

function P:IsMyGuid(name)
    return self.guid == name;
end

function P:BuildPingResponse()
    local level = UnitLevel("player");
    local tal = P:Talents();

    local response = {
        type = TypeApi.Ping.id,
    };

    response.guid = self.guid;
    response.cls = self.classSystem;
    response.lvl = level;
    response.ail = P:GetAverageItemLevel();
    response.tals = tal.text;
    response.v = A.BUILD.appVersion;
    return U:TableToAddonMessage(response);
end

function P:LfmResponse(dungeon)
    local level = UnitLevel("player");
    local role = dungeon.role or P.role or "";
    local note = dungeon.note or P.note or "";
    note = string.gsub(note, '%;', ' ');
    local tal = P:Talents();

    local response = {
        type = TypeApi.Lfg.id,
        v = A.BUILD.appVersion
    };
    response.did = dungeon and dungeon.id or nil;
    response.guid = self.guid;
    response.cls = self.classSystem;
    response.lvl = level;
    -- average item level
    response.ail = P:GetAverageItemLevel();
    -- talents
    response.tals = tal.text;
    response.v = A.BUILD.appVersion;
    response.role = role;
    response.note = string.sub(note, 1, 100);

    --Tabu.dump(response);

    return U:TableToAddonMessage(response);

    -- local items = {
    --     "type="..TypeApi.Lfg.id,
    --     "dungeonId="..dungeon.id,
    --     "guid="..self.guid,
    --     "name="..self.name,
    --     "class="..self.class,
    --     "engClass="..self.classSystem,
    --     "level="..level,
    --     "avgILevel="..P:GetAverageItemLevel(),
    --     "role="..role,
    --     "note="..note
    -- }

    -- local text = U:JoinToString(items, C.RequestAddonSeparator);

    -- return text;
end




-- function P:ParseLfmResponse(text)
--     local parsed = U:TextSplit(text, C.RequestSeparator);
--     local rtype, dungeonId, playerId, playerName, playerClass, playerSystemClass, level, role = unpack(parsed);
--     local note = U:JoinToString(parsed, C.RequestSeparator, 9);

--     local player = DB:GetOrCreate(playerId, "addon", {
--         id = playerId,
--         name = playerName,
--         class = playerSystemClass,
--         level = level,
--     }, true);



--     local res = {
--         id = player.id,
--         type = rtype,
--         dungeonId = dungeonId,
--         role = role,
--         note = note,
--         time = GetTime(),
--         native = true,
--         player = player
--     }

--     return res;
-- end    

-- function P:ParseAddonResponse(text)
--     local parsed = U:TextSplit(text, C.RequestSeparator);
--     local rtype, dungeonId, playerId, playerName, playerClass, playerSystemClass, level, role = unpack(parsed);
--     local note = U:JoinToString(parsed, C.RequestSeparator, 9);
--     local type = TypeApi:GetById(rtype);
--     local player = DB:GetOrCreate(playerId, "addon", {
--         id = playerId,
--         name = playerName,
--         class = playerSystemClass,
--         level = level,
--     }, true);

--     local dungeon = DungeonApi:GetById(dungeonId);

--     return {
--         id = player.id,
--         type = type,
--         dungeon = dungeon,
--         player = player,
--         role = role,
--         note = note,
--         native = true,
--     }
-- end

function P:ChatResponseToLfmResponse(resp, native)

    local dun = D:GetOne(resp.zone, resp.mode);
    if dun == nil then return end

    local locClass, engClass, locRace, engRace, gender, name, server = GetPlayerInfoByGUID(resp.playerId);

    local res = {
        id = resp.playerId,
        name = resp.playerName,
        type = resp.type,

        dungeonId = dun.id,

        class = locClass,
        classSystem = engClass,
        level = nil,

        role = resp.role,
        note = resp.note,
        timestamp = GetTime(),
        native = native,
    
    }

    return res;
end

function P:DetectRole()
    if self.classSystem == "ROGUE" then
        return C.Roles.mdd;
    elseif self.classSystem == "MAGE" or self.classSystem == "HUNTER" then
        return C.Roles.rdd;
    elseif self.classSystem == "PRIEST" then
        local tal = self:Talents();
        if tal.total >= 30 then
            if tal.PriestShadow >= tal.PriestHoly and tal.PriestShadow >= tal.PriestDiscipline then
                return C.Roles.rdd;
            else
                return C.Roles.heal;
            end
        else
            return C.Roles.heal;
        end
    elseif self.classSystem == "WARRIOR" then
        local tal = self:Talents();
        if tal.total >= 20 then
            if tal.WarriorProtection >= 20 then
                return C.Roles.tank;
            else
                return C.Roles.mdd;
            end
        else
            return C.Roles.tank;
        end
    end
end


function P:Talents()
    local talents = {
        counter = {},
        text = "",
        total = 0
    };
    local numTabs = GetNumTalentTabs();
    for t=1, numTabs do
        local name, _, _, sysname = GetTalentTabInfo(t);
        talents[sysname] = 0;
        local numTalents = GetNumTalents(t);

        -- if talents.text ~= "" then
        --     talents.text = talents.text .. "/";
        -- end
        -- talents.text = talents.text .. numTalents;


        for i=1, numTalents do

            nameTalent, icon, tier, column, currRank, maxRank = GetTalentInfo(t,i);
            talents[sysname] = talents[sysname] + currRank;
            talents.total = talents.total + currRank;

            talents.counter[t] = (talents.counter[t] or 0) + currRank;

        end
        --print(sysname, talents[sysname])
    end

    talents.text = U:JoinToString(talents.counter, "/")
    talents.counter = nil;

    --Tabu.dump(talents);

    return talents;
end

function P:GetAverageItemLevel()

    --[[
        0 = ammo
        1 = head
        2 = neck
        3 = shoulder
                4 = shirt
        5 = chest
        6 = waist
        7 = legs
        8 = feet
        9 = wrist
        10 = hands
        11 = finger 1
        12 = finger 2
        13 = trinket 1
        14 = trinket 2
        15 = back
        16 = main hand
        17 = off hand
        18 = ranged

    ]]--


    local itemscount = 17; -- minus shirt
    local levelSumm = 0;
    for i = 1, 18 do

        local itemId = GetInventoryItemID("player", i);
        if itemId ~= nil and i ~= 4 then
            local itemName, itemLink, itemRarity, itemLevel = GetItemInfo(itemId);
            itemLevel = itemLevel or 0;
            levelSumm = levelSumm + itemLevel;
        end
    end
    return ceil((levelSumm / 17) * 100) / 100;
end



function P:Initialize()

    P.name = UnitName("player");
    P.guid = UnitGUID("player");
    P.class, P.classSystem = UnitClass("player");
    P.role = Cache:GetOption("lfg.player.role") or P:DetectRole();
    P.note = Cache:GetOption("lfg.player.note");

end

--P:Talents();