local A = select(2, ...);
local DB = A:Mod("players-db"); -- A.PlayersDB;
local Bus = A:Mod("chat-bus"); --A.ChatBus;
local Cache = A.Cache;
local U = A.Utils;

local ONEDAY_IN_SECONDS = 86400;
local ONEHOUR_IN_SECONDS = 3600;
local TEST_SECONDS = 5;

local PING_DELAY = ONEHOUR_IN_SECONDS;

--C_FriendList

DB.byId = {};
DB.byFullName = {};

function DB:Get(fullName)
    return self.byFullName[fullName];
end

function DB:Set(player)
    self.byFullName[player.fullName] = player;
end


function MyGetPlayerInfoByGUID(id)
    local OK, locClass, engClass, locRace, engRace, gender, name, server = pcall(GetPlayerInfoByGUID, id);
    if OK then
        return locClass, engClass, locRace, engRace, gender, name, server;
    -- else
    --     print("ERROR GUID: ", id)
    end
end


function DB:GetOrCreate(fullName, source, fields, rewrite) 
    
    

    local player = self:Get(fullName);
    local isNew = player == nil;

    if not player then
        player = {
            fullName = fullName,
        }
        if source then
            player.source = source;
        end
    end

    self:MixinCachedPlayer(player);

    if type(fields) == "table" then
        Mixin(player, fields);
    end

    if player.id and not player.infoByGuid then
        --print("- guid", player.id);

        local locClass, engClass, locRace, engRace, gender, name, server = MyGetPlayerInfoByGUID(player.id);

        if not player.gender and gender then
            player.gender = gender;
        end

        if not player.race and race then
            player.race = engRace;
        end
    
        if not player.class and engClass then
            player.class = engClass;
        end
    
        if not player.server and server then
            player.server = server;
        end
    
        if not player.name and name then
            player.name = name;
        end

        player.infoByGuid = true;

    end

    if isNew then
        self.byFullName[player.fullName] = player;
    end

    return player;
end




local EventHandler = CreateFrame("Frame");
function EventHandler:ProcessWhoResposne()
    local returned, total = C_FriendList.GetNumWhoResults();
    for index = 1, returned do
        local info = C_FriendList.GetWhoInfo(index);
        -- TODO: update player info
        --Tabu.print(info);
    end
end

EventHandler:RegisterEvent("WHO_LIST_UPDATE");
EventHandler:SetScript("OnEvent", function(self)
    self:ProcessWhoResposne();
end);

local function UpdateLastPing(player, waitPingResponse, when)
    when = when or GetTime();
    player.lastPing = when;
    player.awaitingPingResponse = waitPingResponse;
end

function DB:TrySendPing(fullName)
    local player = self:Get(fullName);
    local now = GetTime();

    if not player then
        U:Log("player not found for ping");
    end

    if not player.lastPing or (now - player.lastPing) > PING_DELAY then
        --U:Log("pinging: ", player.name);
        Bus:SendPing(player.name);
        UpdateLastPing(player, true, now);
        return true;
    end

end


function DB:RefreshPlayer(_player)

    if not _player then return end

    local player = self:Get(_player.fullName);

    if not player then
        return _player;
    end

    Mixin(player, _player);

    return player;
end


function DB:RegisterPing(_player)

    local player = self:RefreshPlayer(_player)

    if not player then
        U:Log("player not found for ping");
        return;
    end

    UpdateLastPing(player);

    self:RememberAddonUser(player);

    Bus.PingResponses = Bus.PingResponses + 1;
    return player;
end

function DB:RegisterPingResponse(_player)
    self:RegisterPing(_player);
end

function DB:RegisterPingRequest(_player)
    local p = self:RegisterPing(_player);
    Bus:SendPingResponse(p.fullName);
end

function DB:RememberAddonUser(player)
    self:SetCachedPlayer(player);
    U:Log(U:InlineColor("stored", "0080ff"), U:InlineColor(player.fullName, "ff8000"), player.version);
end

function DB:GetCachedPlayer(fullName)
    local players = self:GetCachedPlayers(fullName);
    return players[fullName];
end

function DB:SetCachedPlayer(player)
    local players = self:GetCachedPlayers(player.fullName);
    players[player.fullName] = player;
    return player;
end


function DB:GetCachedPlayers(fullName)
    local users = Cache:GetGlobalOption("addon.users");

    if not users then
        users = {};
        Cache:SetGlobalOption("addon.users", users);
    end

    return users;
end

function DB:MixinCachedPlayer(player)

    local cached = self:GetCachedPlayer(player.fullName);

    if not cached or type(cached) ~= "table" then return end

    Mixin(player, cached);

end

function DB:GetAddonUsersCount()
    local users = Cache:GetGlobalOption("addon.users");
    if not users then return 0 end
    local count = 0;
    for _, users in pairs(users) do
        count = count + 1;
    end
    return count;
end


function TabuLfg:DumpPlayers()
    local users = DB:GetCachedPlayers();
    Tabu.dump(users);
end