local A = select(2, ...);
local R = A:Mod("request"); -- A.Request;
local C = A.Constants;
local Cache = A.Cache;
local ModeApi = A.Api.Mode;
local RoleApi = A.Api.Role;
local U = A.Utils;

local function wrap(text)
    local should = Cache:GetOption("chat.send.addAngleBrackets");
    if not should then return text end;
    return "<"..text..">";
end

local function rolewrap(text)
    local should = RoleApi.tank.name == text or RoleApi.heal.name == text;
    if not should then return text end
    return wrap(text);
end

function R:ToString(model)
    local text = model.type .. C.RequestSeparator .. wrap(model.zone);

    if ModeApi:IsHeroic(model.mode) then
        text = text .. C.RequestModeSeparator .. model.mode;
    end

    if model.role then
        text = text .. C.RequestSeparator .. model.role;
    elseif model.roles then

        local roles = {};
        for role, value in pairs(model.roles) do
            if value == true then
                table.insert(roles, role);
            end
        end


        if #roles > 0 then
            text = text .. C.RequestSeparator;
            for _i = 1, #roles do
                if _i > 1 then
                    text = text .. C.RequestRoleSeparator;
                end
                text = text .. rolewrap(roles[_i]);
            end
        end
    

    end


    if model.note ~= nil then
        text = text .. C.RequestSeparator .. model.note;
    end

    return text;
end

local RolesHash = {}
for _i, hashValue in pairs(C.Roles) do
    RolesHash[hashValue] = _i;
end

local function ParseRoles(_roles)
    local roles = {};
    local was = false;
    if _roles == nil or _roles == "" then return end;

    local arr = U:TextSplit(_roles, C.RequestRoleSeparator);

    for _k, val in pairs(arr) do
        if RolesHash[val] ~= nil then
            table.insert(roles, val);
            was = true;
        end
    end

    if was == true then
        return roles;
    end
end

local function ParseMode(mode)
    if string.lower(mode or "") == string.lower(C.Modes.Heroic[1]) then
        mode = C.Modes.Heroic[1];
    else
        mode = C.Modes.Normal[1];
    end
    return mode;
end

local function ParseNote(request)
    local note = "";
    for _i = 4, #request do
        local chunk = request[_i];

        if chunk == nil then return note end

        if note ~= "" then
            note = note .. C.RequestSeparator;
        end
        note = note .. chunk;
    end
    return note;
end

function R:Parse(request)
    local requestType, _zone, _roles = unpack(request);

    local note = ParseNote(request);

    local zone, mode = unpack(U:TextSplit(_zone, C.RequestModeSeparator));
    
    mode = ParseMode(mode);
    
    local roles = ParseRoles(_roles);

    if _roles ~= nil and roles == nil then
        note = note or _roles;
    end

    local role;
    if roles and #roles == 1 then
        role = roles[1];
    end

    return {
        type = requestType,
        zone = zone,
        mode = mode,
        roles = roles,
        role = role,
        note = note
    }

end


local function WordIsDungeon(word)
end

local function WordIsMode(word)
end

local function WordIsRole(word)
end

local function DetectWord(word, res)

    word = string.lower(word);

    if word == string.lower(C.Lfg) then
        res.type = C.Lfg;
        return true;
    end

    if word == string.lower(C.Lfm) then
        res.type = C.Lfm;
        return true;
    end

    if word == string.lower(C.Lfg) then
        res.type = C.Lfg;
        return true;
    end

    local zone = WordIsDungeon(word);
    if zone then
        res.zone = zone;
        return true;
    end

    local mode = WordIsMode(word);
    if mode then
        res.mode = mode;
        return true;
    end

    local role = WordIsRole(word);
    if role then
        if not res.role then
            res.reloc = role;
        elseif not res.roles then
            res.roles = { res.role, role };
        else
            table.insert(res.roles, role);
        end
        return true;
    end

end

function R:TryToDetect(req)
    local res = {};
    local found = false;
    for _, word in pairs(req) do
        if DetectWord(word, res) then
            found = true;
        end
    end
    if found == true then
        return res;
    end
end