local ANAME, A = select(1, ...);
local U = A.Utils;

--A.ConfigOptions = {};

local O = A:Mod("addon-config");
--A.ConfigOptions;

local PANEL_NAME = ANAME;

local fb = A.Lib.FB;
local Cache = A.Cache;
--local Comp = A.Companion;

local emptyTable = {};

local function get_invoked(arg, ...)
    if type(arg) == "function" then
        return arg(...);
    else
        return arg
    end
end

------------------------------------
------------------------------------

--- controls mixins
local ControlsMixins = {
    GroupName = {
        controlHeight = 30,
    },
    Checkbox = {
        controlHeight = 30,
        Update = function(self)
            self.checkbox:SetChecked(self.options.checked());
            self.text:SetText(get_invoked(self.options.text));
            local width = self:GetWidth() - self.checkbox:GetWidth();
            if width <= 24 then
                width = 200;
            end
            self.checkbox:SetHitRectInsets(0, -width, 0, 0);
        end,
    },
    Number = {
        controlHeight = 30,
        OnChange = function(self)
            local value;
            local text = self.edit:GetText();
            if text == "" then
                value = nil;
            else
                value = tonumber(text);
            end
            self:SetNewValue(value);
        end,
        SetNewValue = function(self, value)
            if value ~= nil or self.nilAllowed then
                self._hasChanges = true;
                self.newValue = value;
            end
        end,

        GetNewValue = function(self)
            if self._hasChanges then
                return self.newValue, true
            end
        end,

        StopEdit = function(self)
            self.edit:ClearFocus();
            self:RestoreValue();
        end,
        OnEsc = function(self)
            self:StopEdit();
        end,
        OnDone = function(self)
            local value, isOK = self:GetNewValue();
            if isOK then
                self:Apply(value);
            end
            self:StopEdit();
        end,
        Apply = function(self, value)
            print("applied:", value);
            self.options.set(value);
        end,
        RestoreValue = function(self)
            self.edit:SetText(self.options.get())
        end
    }
}

------------------------------------
------------------------------------

--- control mixin
local Control = {};

function Control:SetPoints(frame, parent, prev)
    if prev then
        frame:SetPoint("TOPLEFT", prev, "BOTTOMLEFT", 0, 0);
        frame:SetPoint("TOPRIGHT", prev, "BOTTOMRIGHT", 0, 0);
    else
        --print("-- parent is:", parent:GetSize())
        frame:SetPoint("TOPLEFT", parent, "TOPLEFT", 0, 0);
        frame:SetPoint("TOPRIGHT", parent, "TOPRIGHT", 0, 0);
    end
end

function Control:ApplyMixin(frame)
    if type(ControlsMixins[frame.controlType]) == "table" then
        Mixin(frame, ControlsMixins[frame.controlType]);
    end
end

function Control:Setup(frame, options, parent, prev)
    frame.controlType = options.type;
    frame.options = options;

    self:ApplyMixin(frame);
    self:SetPoints(frame, parent, prev);

    fb:SetTooltip(frame, options.tooltip);

end

function Control:Create(options, parent, prev)
    local frame = CreateFrame("Frame", "$parent_"..options.name, parent);

    self:Setup(frame, options, parent, prev);

    if frame.controlHeight then
        frame:SetHeight(frame.controlHeight)
    end
    return frame;
end

------------------------------------
------------------------------------


--- builder api
local Builder = {};



function Builder:RegisterControl(parent, control)
    local controls = parent.controls;
    if not controls then
        controls = {};
        parent.controls = controls;
    end
    table.insert(parent.controls, control);
end

-----------------------------------

function Builder:CreateCheckbox(options, parent, prev)

    local frame = Control:Create(options, parent, prev);

    local check = CreateFrame("CheckButton", "$parent_Checkbox", frame, "ChatConfigCheckButtonTemplate");
    frame.checkbox = check;
    check:SetPoint("LEFT", frame, "LEFT");
    check:SetScript("OnClick", options.click);
    check:SetChecked(options.checked(check));
    check.tooltip = options.tooltip;

    frame.text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    frame.text:SetPoint("LEFT", check, "RIGHT", 0, 0);
    frame.text:SetPoint("RIGHT", frame, "RIGHT", 0, 0);
    frame.text:SetJustifyH("LEFT");
    frame.text:SetText(options.text);
    frame.text:SetTextColor(1,1,1);

    local width = frame:GetWidth() - check:GetWidth();
    check:SetHitRectInsets(0, -200, 0, 0);


    Builder:RegisterControl(parent, frame);

    return frame;
end


function Builder:CreateGroupName(options, parent, prev)

    local frame = Control:Create(options, parent, prev);
    frame.header = frame:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
    
    local hfs = frame.header;
    hfs:SetPoint("LEFT", frame, "LEFT", 0, 0);
    hfs:SetJustifyH("LEFT");
    hfs:SetText(options.header);
    hfs:SetTextColor(1, .8, 0);

    return frame;
end

function Builder:CreateNumber(options, parent, prev)

    local frame = Control:Create(options, parent, prev);

    local edit = CreateFrame("EditBox", "$parent_EditBox", frame, "InputBoxInstructionsTemplate");
    frame.edit = edit;

    edit:SetPoint("LEFT", frame, "LEFT", 0, 0);
    local height = frame:GetHeight() - 10;
    edit:SetHeight(height);
    edit:SetWidth(40);
    edit.Middle:SetHeight(height);
    edit.Left:SetHeight(height);
    edit.Right:SetHeight(height);
    edit:SetAutoFocus(false);
    edit:SetJustifyH("RIGHT");
    edit:SetTextInsets(5,5,5,5);
    edit:SetTextColor(1,1,0);
    U:UpdateFontString(edit, nil, 12);

    edit:SetText(options.get());

    edit:SetScript("OnEnterPressed", function() frame:OnDone() end);
    edit:SetScript("OnEscapePressed", function() frame:OnEsc() end);
    edit:SetScript("OnKeyUp", function() frame:OnChange() end);

    local t = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    t:SetPoint("LEFT", edit, "RIGHT", 5, 0);
    t:SetText(options.text);
    t:SetHeight(height);
    t:SetTextColor(1,1,1);
    return frame;
    --e:SetText(value);    
    --U:UpdateFontString(e, nil, 12);
    -- e:SetScript("OnEnterPressed", function() f:OnApply() end);
    -- f.edit = e;

    -- e:SetScript("OnEscapePressed", function() f:OnEsc() end);
end

function Builder:CreateNumberNew(options, parent, prev)

    local frame = Control:Create(options, parent, prev);
    local height = frame:GetHeight() - 10;

    local t = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    t:SetPoint("LEFT", frame, "LEFT", 5, 0);
    t:SetText(options.text);
    t:SetHeight(height);
    t:SetTextColor(1,1,1);

    local edit = CreateFrame("EditBox", "$parent_EditBox", frame, "InputBoxInstructionsTemplate");
    frame.edit = edit;

    edit:SetPoint("LEFT", t, "RIGHT", 5, 0);
    edit:SetHeight(height);
    edit:SetWidth(40);
    edit.Middle:SetHeight(height);
    edit.Left:SetHeight(height);
    edit.Right:SetHeight(height);
    edit:SetAutoFocus(false);
    edit:SetJustifyH("RIGHT");
    edit:SetTextInsets(5,5,5,5);
    edit:SetTextColor(1,1,0);
    U:UpdateFontString(edit, nil, 12);

    edit:SetText(options.get());

    edit:SetScript("OnEnterPressed", function() frame:OnDone() end);
    edit:SetScript("OnEscapePressed", function() frame:OnEsc() end);
    edit:SetScript("OnKeyUp", function() frame:OnChange() end);


    return frame;
    --e:SetText(value);    
    --U:UpdateFontString(e, nil, 12);
    -- e:SetScript("OnEnterPressed", function() f:OnApply() end);
    -- f.edit = e;

    -- e:SetScript("OnEscapePressed", function() f:OnEsc() end);
end

-----------------------------------

function Builder:PopulateOptions(options, parent, prev)
    for _, option in pairs(options) do
        local methodName = "Create"..option.type;
        if option.type and Builder[methodName] then
            prev = Builder[methodName](Builder, option, parent, prev);
        else
            U:Log("missing Builder:"..methodName, "for", option.type);
        end
    end
    return prev;
end


-----------------------------------------------
-----------------------------------------------


local builders = {

    -- chat
    function(f, prev)
        local options = A:Mod("chat-frame"):GetConfigOptions();
        return Builder:PopulateOptions(options, f, prev);
    end,

    -- companion
    function(f, prev)
        local options = A.Companion:GetConfigOptions();
        return Builder:PopulateOptions(options, f, prev);
    end,

    

}


local function PopulateOptions(options, parent)
    local prev;
    for _, option in pairs(options) do

        if option.type and Builder["Create"..option.type] then
            prev = Builder["Create"..option.type](Builder, option, parent, prev);
        end

        -- if option.type == "Checkbox" then
        --     prev = Builder:CreateCheckbox(option, parent, prev);
        -- elseif option.type == "GroupName" then
        --     prev = Builder:CreateHeader(option, parent, prev);
        -- end
    end
end

local function PanelContent_OnShow(self)
    local controls = self.controls or emptyTable;
    for _, control in pairs(controls) do
        if control.Update then
            control:Update();
        end
    end
end

local PanelMixin = {
    okay = function()
        A:ShowMainFrame();
    end,
    cancel = function()
        A:ShowMainFrame();
    end,
}

function O:OnInitialize()
    if self.frame then return end

    local panel = CreateFrame("FRAME", "TabuLfgOptionsFrame", UIParent);
    Mixin(panel, PanelMixin);
    self.frame = panel;


    panel.name = PANEL_NAME;
    panel:Hide();
    InterfaceOptions_AddCategory(panel);
    local panelContent = CreateFrame("Frame", "$parent_Content", panel);
    panelContent:SetPoint("TOPLEFT", panel, "TOPLEFT", 10, -10);
    panelContent:SetPoint("BOTTOMRIGHT", panel, "BOTTOMRIGHT", -10, 10);
    
    fb:ApplyFrameBuilders(panelContent, builders);
    panelContent:SetScript("OnShow", PanelContent_OnShow);

end

function O:Show()
    A:HideMainFrame();
    O:Start();
    InterfaceOptionsFrame_Show();
    InterfaceOptionsFrame_OpenToCategory(PANEL_NAME);
end