local A = select(2, ...);
local U = A.Utils;

local ModMixin = {

	Initialize = function(self, ...)
		if self._initialized then return end
		self._initialized = true;
		if self.OnInitialize then
			self:OnInitialize(...)
		end
	end,

	Start = function(self, ...)
        self:Initialize();
		if self.started then return end
		self.started = true;
		if self.OnStart then
            self:Log("starting");
			self:OnStart(...);
            self:Log("started");
		end
	end,

	Stop = function(self, ...)
		if self.started ~= true then return end
		self.started = false;
		if self.OnStop then
			self:OnStop(...);
		end
	end,

    Restart = function(self, ...)
        self:Stop(...);
        self:Start(...);
    end,

    On = function(self, event, callback, context)
        context = context or self;
        self.RegisterCallback(context, event, callback);
    end,

    Off = function(self, event, context)
        context = context or self;
        self.UnregisterCallback(context, event);
    end,

    ListenTo = function(self, entity, event, callback)
        entity:On(event, callback, self);
    end,

    StopListenTo = function(self, entity, event)
        entity:Off(event, self);
    end,

    Trigger = function(self, event, ...)
        self.events:Fire(event, ...);
    end,
    Log = function(self, ...)
        if not self.verbose then return end;
        U:Log(U:InlineColor(self.name or "noname", "906090"), ...);
    end,
}


local function GetThing(store, key, onCreate)
    local thing = store[key];
    if not thing then
        thing = {
            name = key,            
        }
        if onCreate then
            onCreate(thing);
        end
        store[key] = thing;
    end
    return thing;
end

local function onCreateModule(mod)
    Mixin(mod, ModMixin);
    mod.events = LibStub("CallbackHandler-1.0"):New(mod);
end

A.Mods = {}
A.Mod = function(self, key)
	return GetThing(self.Mods, key, onCreateModule);
end

A.Components = {}
A.Com = function(self, key)
    return GetThing(self.Components, key);
end