local A = select(2, ...);
local U = A.Utils;
local fb = A.Lib.FB;
local C = A.Constants;
local L = A.Localize.translate;

function U:TextStartsWith(text, starts)
    return string.sub(text, 1, string.len(starts)) == starts;
end



function U:TextSplit(text, sep)
    
    if sep == nil then
        sep = "%s"
    end

    local t = {};

    for str in string.gmatch(text, "([^"..sep.."]+)") do
            table.insert(t, str)
    end

    return t;

end



----------------------




-- function U:UpdateFauxScroll()

-- 	local scroll = self;
-- 	local parentFrame = scroll:GetParent();
-- 	local options = scroll.options;

-- 	local offset = FauxScrollFrame_GetOffset(scroll);
-- 	local lines = options.GetLinesCount(parentFrame);


-- 	FauxScrollFrame_Update(scroll, lines, options.buttonsCount, options.buttonHeight, nil, nil, nil, nil, nil, nil, true);

-- 	for _i = 1, options.buttonsCount do
-- 		options.UpdateButton(parentFrame, _i, offset + _i);
-- 	end

-- end

-- function U:OnVerticalScroll(offsetInPx)
-- 	FauxScrollFrame_OnVerticalScroll(self, offsetInPx, self.options.buttonHeight, lfb.UpdateFauxScroll);
-- end


-------------------------------
-- FAUX
-------

local BuildFauxScrollFrameOptions = {
    globalName = "$parentFaux",
    parentKey = "Scroll",
    points = {
        { "TOPLEFT", "parent", "TOPLEFT", 0, 0 },
        { "BOTTOMRIGHT", "parent", "BOTTOMRIGHT", 0, 0 }
    },
	textures = {
		{
			globalName = nil,
			layer = "ARTWORK",
			inheritsFrom = nil,
			parentKey = "textureTop",
			texture = "Interface\\PaperDollInfoFrame\\UI-Character-ScrollBar",
			texCoord = { 0, 0.484375, 0, .2 },
			point = { "TOPRIGHT", "parent", "TOPRIGHT", 0, 0 },
			width = 31,
			height = 50,	
		},
		{
			globalName = nil,
			layer = "ARTWORK",
			inheritsFrom = nil,
			parentKey = "textureBottom",
			texture = "Interface\\PaperDollInfoFrame\\UI-Character-ScrollBar",
			point = { "BOTTOMRIGHT", "parent", "BOTTOMRIGHT", 0, -19 },
			texCoord = { 0.515625, 1, .3, .495 },
			width = 31,
			height = 50,
		},
		{
			globalName = nil,
			layer = "ARTWORK",
			inheritsFrom = nil,
			parentKey = "textureBottom",
			texture = "Interface\\PaperDollInfoFrame\\UI-Character-ScrollBar",
			points = { { "TOPRIGHT", "parent", "TOPRIGHT", 0, -49 }, { "BOTTOMRIGHT", "parent", "BOTTOMRIGHT", 0, 22 } },
			texCoord = { 0, 0.484375, .4, .8 },
			width = 31,
		},
	}
}


local FauxScroll = {}

function FauxScroll:UpdateScrollButtons()

	local scroll = self;
	local parentFrame = scroll:GetParent();
	local options = scroll.options;

	local offset = FauxScrollFrame_GetOffset(scroll);
	local lines = parentFrame:GetDataCount();
    local buttonsCount = #parentFrame.Buttons;
    local buttonHeight = scroll.buttonHeight;

	FauxScrollFrame_Update(scroll, lines, buttonsCount, buttonHeight, nil, nil, nil, nil, nil, nil, true);

	for index = 1, buttonsCount do
        parentFrame:UpdateButton(index, offset + index)
	end

end

function FauxScroll:OnVerticalScroll(offsetInPx)
	FauxScrollFrame_OnVerticalScroll(self, offsetInPx, self.buttonHeight, self.UpdateScrollButtons);
end


local FauxFrameMixin = {
    -- This method should be defined on certain frame
    -- GetData = function(self)
    --     return {};
    -- end,

    GetDataCount = function(self)
        local data = self:GetData();
        return data and #data or 0;
    end,

    GetDataItem = function(self, index)
        local data = self:GetData();
        return data[index];
    end,

    GetButtons = function(self)
        return self.Buttons;
    end,

    GetButton = function(self, index)
        local btns = self:GetButtons();
        return btns[index];
    end,

    -- GetLinesCount = function(self)
    --     local data = self:GetData();
    --     return #data;
    -- end,

    UpdateButton = function(self, buttonIndex, itemIndex) 
        --print("U:", buttonIndex, itemIndex);
        local button = self:GetButton(buttonIndex);
        local item = self:GetDataItem(itemIndex);
        if not item then
            button:Hide();
        else
            button:Show();
            button:Update(item, itemIndex, buttonIndex);
        end        
    end,



    UpdateButtons = function(self)
        return self.Scroll:UpdateScrollButtons();
        --return U.UpdateFauxScroll(self.Scroll);
    end,
}


function U:CreateScrollButtons(parent, buttonsCount, CreateButton)
    parent.Buttons = {}
    local prev;
    for index = 1, buttonsCount do
        prev = CreateButton(index, parent, prev);
        --prev:Hide();
        table.insert(parent.Buttons, prev);
    end
    return parent.Buttons;
end

function U:BuildFauxScrollFrame(frame, buildOptions, buttonHeight)
    local scrl;

    local scrl = CreateFrame("ScrollFrame", buildOptions.globalName, frame, "FauxScrollFrameTemplate");

    Mixin(scrl, FauxScroll);

    scrl.buttonHeight = buttonHeight;

    fb:SetupChild(frame, buildOptions, scrl);
    fb:SetupSize(scrl, buildOptions, frame);
	fb:SetupAnchor(scrl, buildOptions, frame);
	fb:SetupTextures(scrl, buildOptions, frame);

	local sb = _G[scrl:GetName() .. "ScrollBar"];
	sb:ClearAllPoints();
	sb:SetPoint("TOPRIGHT", scrl, "TOPRIGHT", -7, -20);
	sb:SetPoint("BOTTOMRIGHT", scrl, "BOTTOMRIGHT", -7, 20);
    scrl:SetScript("OnVerticalScroll", scrl.OnVerticalScroll);

    return scrl;
end

function U:AddFauxScroll(frame, buttonsCount, CreateButton, FrameMixin)

    local buttons = U:CreateScrollButtons(frame, buttonsCount, CreateButton);
    local buttonHeight = buttons[1]:GetHeight();

    Mixin(frame, FauxFrameMixin, FrameMixin);

	U:BuildFauxScrollFrame(frame, BuildFauxScrollFrameOptions, buttonHeight);

end

-------------------------

function U:GetModeName(modeShort)
    for _, tbl in pairs(C.Modes) do
        if tbl[1] == modeShort then
            return tbl[2];
        end
    end
    return "";
end

--------------------

function U:UpdateFontString(fs, font, size, options)
    local fnt = { fs:GetFont() };

    if font then
        fnt[1] = font;
    end

    if size then
        fnt[2] = size;
    end

    if options then
        fnt[3] = options;
    end

    fs:SetFont(unpack(fnt));

end



--------------------

function U:CreateDropdownInitializer(dropdown, options, parent)
    return function()
        options.buildMenu(dropddown, parent);
    end;
end

function U:CreateDropdownMenu(frame, options)

    local dropdown = CreateFrame("Frame", "$parent_DropdownMenu", frame, "UIDropDownMenuTemplate")
    fb:SetupFrame(dropdown, options, frame);
    local initialize = self:CreateDropdownInitializer(dropdown, options, frame);
    --UIDropDownMenu_Initialize(dropdown, initialize, "MENU");
    frame:SetScript("OnClick", function(self, button) 
        ToggleDropDownMenu(1, nil, dropdown, "cursor", 3, -3);        
    end);

end


-------------------

function U:JoinToString(tbl, separator, startFrom)
    separator = separator or ",";
    local text = "";
    startFrom = startFrom or 1;
    for i = startFrom, #tbl do
        local v = tbl[i];

        if v ~= nil and v ~= "" then
            if text ~= "" then
                text = text .. separator;
            end
            text = text .. v;
        end

    end
    return text;
end


------------------------


function U:GetLFGChannelName()
    local channels = { EnumerateServerChannels() };
    local lfg = table.remove(channels);

    local id, name = GetChannelName(lfg);
    if id == nil or id == 0 then
        print("|cffff0000Tabu-LFG error: You are not joined to channel \""..lfg.."\", type in chat: /join " .. lfg .. "|r");
    end
    return id, name
end

function U:GetGeneralChannelName()
    local channels = { EnumerateServerChannels() };
    local channelIndex = channels[1];
    local id, name = GetChannelName(channelIndex);
    return id, name
end

------------------------

local function NoOp()
end

local EditTextFrameMixin = {
    Setup = function(self, value, OnDone, args, header, apply)
        self.args = args or {};
        self.edit:SetText(value or "");
        self.header:SetText(header or "Edit");
        self.apply:SetText(apply or "Apply");

        self.OnDone = OnDone or NoOp;
        
    end,
    OnApply = function(self)
        local value = self.edit:GetText();
        if value == "" then
            value = nil;
        end

        self.OnDone(value, unpack(self.args));

        self:Hide();
    end,
    OnEsc = function(self)
        self:Hide();
    end,

}

local EditTextFrameBuilders = {

    function(f)
        local h = f:CreateFontString("$parent_Header", "OVERLAY", "GameFontNormalSmall");
        h:SetPoint("TOPLEFT", f, "TOPLEFT", 17, -10);
        h:SetWidth(240);
        h:SetJustifyH("LEFT");
        h:SetText("Edit text");
        f.header = h;
        return f;
    end,

    function(f)
        local e = CreateFrame("EditBox", "$parent_EditBox", f, "InputBoxInstructionsTemplate");
        e:SetPoint("TOPLEFT", f, "TOPLEFT", 17, -30);
        e:SetPoint("TOPRIGHT", f, "TOPRIGHT", -10, -30);
        --name:SetSize(123, 25);
        local nameHght = 30;
        e:SetHeight(nameHght);
        e.Middle:SetHeight(nameHght);
        e.Left:SetHeight(nameHght);
        e.Right:SetHeight(nameHght);
        --name:SetAutoFocus(false);
        e:SetText(f.value or "");
        U:UpdateFontString(e, nil, 12);
        e:SetScript("OnEnterPressed", function() f:OnApply() end);
        f.edit = e;

        e:SetScript("OnEscapePressed", function() f:OnEsc() end);

        return name;        
    end,

    function(f)
        local b = CreateFrame("Button", "$parent_Apply", f, "UIPanelButtonTemplate");
        b:SetText("Apply");
        b:SetScript("OnClick", function() f:OnApply() end);
        b:SetPoint("BOTTOM", f, "BOTTOM", 0, 13);
        b:SetWidth(100);
        --local fs = lfgb.Text:GetFont()
        U:UpdateFontString(b.Text, nil, 10);
        f.apply = b;

        

    end,
}

local function BuildEditTextFrame()
    if TabuPopupEditTextFrame ~= nil then return TabuPopupEditTextFrame end
    local f = CreateFrame("Frame", "TabuPopupEditTextFrame", UIParent, "UIPanelDialogTemplate");
    f:Hide();
    f:SetFrameStrata("DIALOG");
    f:SetPoint("CENTER");
    f:SetSize(300, 100);
    fb:MakeMovable(f);
    fb:ApplyFrameBuilders(f, EditTextFrameBuilders);
    Mixin(f, EditTextFrameMixin);
    return f;
end

function U:EditText(value, OnDone, args, header, apply)
    local f = BuildEditTextFrame();
    f:Setup(value, OnDone, args, header, apply);
    f:Show();
end

--U:EditText();

-------------------

function U:Debounce(callback, seconds)

    local handle;
    local debounced = function(...) 
        local args = { ... };
        if handle then
            handle:Cancel();
        end
        handle = C_Timer.NewTimer(seconds, function() callback(unpack(args)) end);
    end;

    return debounced;
end


-------------------


function U:CreateThrottler(GetValueTime, storeForInSeconds, cleanEachInSeconds, name)

    local storeForFunc = type(storeForInSeconds) == "function" and storeForInSeconds or nil;
    local cleanEachFunc = type(cleanEachInSeconds) == "function" and cleanEachInSeconds or nil;

    storeForInSeconds = storeForInSeconds or 60;
    cleanEachInSeconds = cleanEachInSeconds or 30;

    local throttler = {
        name = name,
        cache = {},

        storeForInSeconds = storeForInSeconds, 
        cleanEachInSeconds = cleanEachInSeconds,
        _GetStoreFor = storeForFunc,
        _GetCleanEach = cleanEachFunc,

        GetStoreFor = function(self)
            if self._GetStoreFor then
                return self._GetStoreFor();
            else
                return self.storeForInSeconds;
            end
        end,

        GetCleanEach = function(self)
            if self._GetCleanEach then
                return self._GetCleanEach();
            else
                return self.cleanEachInSeconds;
            end
        end,

        _GetValueTime = GetValueTime,

        Add = function(self, key, value)
            self.cache[key] = value;
        end,

        Exist = function(self, key, value)
            return self.cache[key] ~= nil;
        end,

        TryAdd = function(self, key, value)

            -- do not throttle if stopped
            if not self.started == true then return true end

            local storeFor = self:GetStoreFor();

            if storeFor == 0 then return true end

            if self.cache[key] ~= nil then return false end
            
            self.cache[key] = GetTime();

            return true;

        end,

        Wipe = function(self)
            local now = GetTime();
            local total, remains, removed = 0, 0, 0;
            local cleanEach = self:GetCleanEach() or 0;

            for key, time in pairs(self.cache) do
                total = total + 1;
                --local time = self._GetValueTime(val);
                if time == nil or (now - time) > cleanEach then
                    removed = removed + 1;
                    self.cache[key] = nil;
                else
                    remains = remains + 1;
                end
            end

            if self.OnWipe then
                self:OnWipe(remains, total);
            end
            --self:Log("WIPE", remains, removed, total);
            return remains, total;
        end,

        Log = function(self, ...)
            U:Log((self.name or "") .. " throttler:", ...);
        end,

        Start = function(self)
            if self.started == true then return end            
            self.started = true;
            local cleanEach = self:GetCleanEach() or 0;
            if cleanEach == 0 then return end
            local api = self;
            self._timer = C_Timer.NewTicker(cleanEach, function()  api:Wipe() end);
        end,

        Stop = function(self)
            self.started = false;
            wipe(self.cache);
            if self._timer then
                self._timer:Cancel();
                self._timer = nil;
            end

        end,

        
    };

    --U:Log("initialized throttler:", name, storeForInSeconds, cleanEachInSeconds);
    return throttler;

end


-------
function U:RgbToHex(tbl)
    local r, g, b, a = unpack(tbl);
    if not a then
        return format("%02x%02x%02x", r*255, g*255, b*255);
    else
        return format("%02x%02x%02x%02x", a*255, r*255, g*255, b*255);
    end
end

function U:InlineColor(text, color, alpha)
    if not color then return text end
    if type(color) == "table" then
        color = self:RgbToHex(color);
    end
    local inline = "ffffffe0";
    color = color or inline;
    local clrlen = string.len(color);
    if clrlen == 8 then
        inline = color
    else
        alpha = alpha or "ff";
        inline = alpha .. color
    end
    return "|c"..inline..(text or "").."|r";
end


----------


local function EasyMenu_Initialize( frame, level, menuList )
	for index = 1, #menuList do
		local value = menuList[index]
		if (value.text) then
			value.index = index;
			UIDropDownMenu_AddButton( value, level );
        elseif value.separator then
            UIDropDownMenu_AddSeparator(level);
		end
	end
end

function U:EasyMenu(menuList, menuFrame, anchor, x, y, displayMode, autoHideDelay )
	if ( displayMode == "MENU" ) then
		menuFrame.displayMode = displayMode;
	end
	UIDropDownMenu_Initialize(menuFrame, EasyMenu_Initialize, displayMode, nil, menuList);
	ToggleDropDownMenu(1, nil, menuFrame, anchor, x, y, menuList, nil, autoHideDelay);
end

----------------

function U:Log(...)
    if not A.BUILD.debugEnabled then return end;
    print(...);
end

function U:Msg(...)
    print(self:InlineColor("Tabu-Lfg:", "00a0ff"), ...);
end

----------------

function U:TableToAddonMessage(tbl)

    local text = "";

    for key, value in pairs(tbl) do
        if value and value ~= "" then
            if text ~= "" then
                text = text .. C.RequestAddonSeparator;
            end
            text = text .. key .. "=" .. value;
        end
    end

    if text == "" then
        text = nil;
    end

    return text;
end

function U:AddonMessageToTable(text)
    
    local kv_pairs = U:TextSplit(text, C.RequestAddonSeparator);

    local msg = {};
    local keys = 0;

    for _, kv_pair_text in pairs(kv_pairs) do
        local kv_pair = U:TextSplit(kv_pair_text, "=");
        if #kv_pair > 2 then
            local mixed = U:JoinToString(kv_pair, "=", 2);
            kv_pair[2] = mixed;
        end
        local value = kv_pair[2];
        if value and value ~= "" then
            keys = keys + 1;
            msg[kv_pair[1]] = value;
        end
    end

    return msg, keys;
end

-------------------


function U:EnableTooltip(frame)
	frame:EnableMouse();
	fb:AddScript(frame, "OnEnter", function(self) 
        if type(self.ShowTooltip) == "function" then
            self:ShowTooltip();
        elseif self.tooltip then
            GameTooltip:SetOwner(self);
            GameTooltip:SetText(self.tooltip);
            GameTooltip:Show();
        end
	end);
	fb:AddScript(frame, "OnLeave", function() 
		GameTooltip:Hide();
	end);
end


------------------

function U:WillBeSentTooltip(frame, text)
    GameTooltip:SetOwner(frame);
    GameTooltip:SetText(U:InlineColor(L("Will be sent in LFG channel:"), "ffffff"));
    GameTooltip:AddLine(text);
    GameTooltip:Show();
end

-----------------

function U:CreateArrowButton(globalName, parent, point, size)
    size = size or 24;
    local btn = CreateFrame("Button", globalName, parent);
    btn:SetSize(size,size);

    btn:SetPoint(unpack(point));

    local nt = btn:CreateTexture(nil, "OVERLAY");
    nt:SetTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Up]]);
    nt:SetSize(size,size);
    nt:SetPoint("RIGHT");
    btn:SetNormalTexture(nt);

    local pt = btn:CreateTexture(nil, "OVERLAY");
    pt:SetTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Down]]);
    pt:SetSize(size,size);
    pt:SetPoint("RIGHT");
    btn:SetPushedTexture(pt);

    local dt = btn:CreateTexture(nil, "OVERLAY");
    dt:SetTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Disabled]]);
    dt:SetSize(size,size);
    dt:SetPoint("RIGHT");
    btn:SetDisabledTexture(dt);

    local ht = btn:CreateTexture(nil, "OVERLAY");
    ht:SetTexture([[Interface\Buttons\UI-Common-MouseHilight]]);
    ht:SetBlendMode("ADD");
    ht:SetSize(size,size);
    ht:SetPoint("RIGHT");
    btn:SetHighlightTexture(ht);
    return btn;
end